function [results]=automatic(grocer_namey,varargin)
 
// PURPOSE: automatic selection of a model by least-squares
// regressions as in Hendry and Krozlic: starting from the
// list of exogenous variables provided by the user, the
// function select the "best" regression, that is one where
// all variables are significant, all specification tests are
// passed, and, if more than one regression has these
// properties, the one with the smallest aic criterion
// ------------------------------------------------------------
// REFERENCES:
// Krolzig, H.-M. and Hendry, D.F. (2001): "Computer Automation
//  of General-to-Specific Model Selection Procedures", Journal
// of Economic Dynamics and Control, 25 (6-7), 831-866.
// ------------------------------------------------------------
// INPUT:
// * grocer_namey = a time series, a real (nx1) vector or a
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector
//   . a string equal to the name of a time series or a (nx1)
//     real vector between quotes
//   . the string 'prt=opt1,opt2,...,optn'
//   where opti is one of the following available options:
//     - nothing (nothing printed !)
//     - initial (results of initial model printed)
//     - st0_mod (results of stage 0 model printed)
//     - st1_mod (results of stage 1 models printed)
//     - st1_union (results of stage 1 union model printed)
//     - st2_mod (results of stage 2 models printed)
//     - st2_union (results of stage 2 union model printed)
//     - final (results of final model printed)
//     - test_inter (results of specification tests for the
//       intermediate results printed)
//     - test_final (results of specification tests for the
//       final model printed)
//     - test (specification tests printed for every regression
//       result printed)
//     - st1_path (results of stage 1 paths printed)
//     - st2_path (results of stage 2 paths printed)
//     - all (all results printed: can be very long !)
//   . the string 'test=opt1,opt2,...,optn'
//   where opti is one of the following available options:
//     - chowtest(p)
//     - predfailin(p)
//     - doornhans
//     - bpagan(p)
//     - hetero_sq
//   . the string 'strategy=liberal' or 'strategy=conservative'
//     if the user wants to use a predefined strategy
//     (default = liberal)
//   . the string 'alpha=p' for the simplification significance
//     level (default = set by the chosen strategy)
//   . the string 'f0_tdo=p' for the top_down pre-test significance
//     level (default = set by the chosen strategy)
//   . the string 'f0_bup=p' for the bottom-up pre-test significance
//     level (default = set by the chosen strategy)
//   . the string 'eta=p' for the specification tests significance
//     level (default = 0.01)
//   . the string 'gam=p' for the F-tests significance level
//     (default = set by the chosen strategy)
//   . the string 'groups_pval=m1;m2;...;mp' where the mi's
//     are thresholds: coefficient whose p-values are greater
//     than a given threshold are gathered to form a model included
//     in stage 1 models
//     (default = set by the chosen strategy)
//   . the string 'crit=x' where x= aic, bic, or hq(default bic)
//   . the string 'test=x1,...,xp' where xi is the name of a
//     test function
//   . the string 'newname(x1,x2)' where x1 is the name of a
//     test function and x2 is the corresponding name the user
//     wants the program to display
//   . the string 'comp=x1;...;xn' where xi is the name of a
//     variable that must be in the regression whatever
//     significance it has
//   . the string 'dropna' if the user wants to remove the NA
//      values from the data
//   . the string 'estim=nwest' if the user wants to use the
//     Newey-West correction of standard errors
//   . the string 'win=xx' where xx is the length of the
//     Bartlett window (for the Newey-West estimation)
// ------------------------------------------------------------
// OUTPUT:
// a results tlist with the following fields:
// * results('meth') = 'automatic'
// * results('estim') = estimation method (ols or nwest)
// * results('strategy') = estimation strategy (liberal,
//   conservative or expert)
// * results('alpha') = simplification significance
//     level (default = set by the chosen strategy)
// * results('f0_tdo') = top_down pre-test significance
//     level (default = set by the chosen strategy)
// * results('f0_bup') = bottom-up pre-test significance
//     level (default = set by the chosen strategy)
// * results('gam') = F-tests significance level
//     (default = set by the chosen strategy)
// * results('eta') = specification tests significance
//     level (default = 0.01)
// * results('comp') = matrix of values for the compulsory
//   variables
// * results('f_test') = the function used to perform the
//                      specification tests
// * results('m_test') = the names of the specification tests
// * results('initial model') = the estimation results of the
//                            unrestricted model
// * results('ending reason') = the reason why the final model
//                            has been chosen
// * results('final model') = the estimation results of the
//                            final model
// * results('wind length') = the size of the Bartlett window
//   (if the Nwest correction has been used)
// * results('top-down model') = the top-down model
// * results('bottom-up model') = the bottom-up model
// * results('stage 0 model') = the estimation results of the
//                             stage 0 restricted model (all
//                             variables whose individual tstat
//                             level is lower than 1 and whose
//                             joint significance level is
//                             lower than f0_tdo are withdrawn)
// * results('selected stage 0 model') = the way the stage 0
//   model has been selected
// * results('checking process model') = the model resulting
//   from the group checking process
// * results('stage i models') = the estimation results of the
//   stage i (i=1, 2.0, 2.1) models and the corresponding paths
// * results('stage i union model') = the estimation results of
//                            the model built from the union of
//                            stage i (i=1, 2.0, 2.1) models
// ------------------------------------------------------------
// PRINTS: results along the options given by the user
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002-2007
// http://dubois.ensae.net/grocer.html
 
 
 
 
grocer_libtab=[0.125 0.075 0.075 0.05  0.025 ; // F tests
               0.1   0.05  0.05  0.025 0.01 ;  // t tests
               0.75  0.5   0.375 0.25  0.1 ;   // top-down F tests
               0.6   0.4   0.3   0.2   0.075   // bottom-up F tests
               ]
 
grocer_constab=[0.05 0.02  0.015 0.01  0.002 ; // F tests
               0.025 0.01  0.01  0.005 0.001 ;  // t tests
               0.5   0.25  0.10  0.05  0.025 ;   // top-down F tests
               0.25  0.125 0.05  0.025 0.0125   // bottom-up F tests
               ]
 
// set default values
grocer_dropna=%f
grocer_strategy='liberal'
grocer_lprt=emptystr()
grocer_ltest=emptystr()
grocer_descent=5
grocer_crit='bic'
done=%f
grocer_flagz=%f
grocer_names=tlist(['names';'jbnorm';'doornhans';'hetero_sq';...
     'chowtest';'predfailin';'arlm';'arch'],'Jarque & Bera',...
     'Doornik & Hansen','hetero x_squared','Chow',...
     'Chow pred. fail. ','AR','ARCH')
grocer_groups_pval=[]
grocer_wreliab=[0.3 ; 0.6 ; 0.4 ; 0.7]
grocer_expert=%f
grocer_estim='ols'
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      grocer_var=strsubst(varargin(grocer_i),' ','')
      grocer_lvar=length(grocer_var)
      str3=part(grocer_var,[1:3])
      str4=part(grocer_var,[1:4])
      str5=part(grocer_var,[1:5])
      str6=part(grocer_var,[1:6])
      str7=part(grocer_var,[1:7])
      str9=part(grocer_var,[1:9])
      if str3 == 'eta' | str3 == 'gam' | str6 == 'f0_tdo' | str5 == 'alpha' then
         grocer_expert=%t
         execstr('grocer_'+grocer_var)
         varargin(grocer_i)=null()
 
      elseif str7 == 'descent' then
         execstr('grocer_'+grocer_var)
         varargin(grocer_i)=null()
 
      elseif str9 == 'strategy=' then
         grocer_strategy=part(grocer_var,10:grocer_lvar)
         varargin(grocer_i)=null()
         grocer_expert0=%f
 
      elseif part(grocer_var,[1:12]) == 'groups_pval=' then
         grocer_groups_pval=evstr(str2vec(grocer_var))
         varargin(grocer_i)=null()
 
      elseif str5 == 'comp=' then
         grocer_comp=str2vec(grocer_var)
         grocer_flagz=%t
         varargin(grocer_i)=null()
 
      elseif str6 == 'estim=' then
         grocer_estim=part(grocer_var,7:length(grocer_var))
         execstr('grocer_estim1='+grocer_estim+'1a')
         execstr('grocer_estim2='+grocer_estim+'2a')
         execstr('grocer_estim3='+grocer_estim+'3a')
         varargin(grocer_i)=null()
 
      elseif str4 == 'win=' then
         execstr('grocer_'+grocer_var')
         varargin(grocer_i)=null()
 
      elseif str4 == 'test' then
         grocer_ltest=grocer_var
         varargin(grocer_i)=null()
 
      elseif str4 == 'crit' then
         grocer_crit=part(grocer_var,strindex(grocer_var,'=')+1:grocer_lvar)
         if grocer_crit ~= 'aic' & grocer_crit ~= 'bic' &...
           grocer_crit ~= 'hq' then
            error('not an available information criterion: '+grocer_crit)
         end
         varargin(grocer_i)=null()
 
      elseif str3 == 'prt' then
         grocer_lprt=grocer_var
         varargin(grocer_i)=null()
 
      elseif str7 == 'newname' then
         indcom=strindex(grocer_var,',')
         grocer_names(1)($+1)=part(grocer_var,strindex(grocer_var,'(')+1:indcom-1)
         grocer_names($+1)=part(grocer_var,indcom+1:strindex(grocer_var,')')-1)
         varargin(grocer_i)=null()
 
      elseif grocer_var == 'dropna' then
         grocer_dropna=%t
         varargin(grocer_i)=null()
      end
 
   elseif typeof(varargin(grocer_i)) ~= 'constant' &...
   typeof(varargin(grocer_i)) ~= 'ts' then
      error('wrong type for entry number '+string(grocer_i+2))
   end
end
 
 
if grocer_flagz then
   [grocer_y,grocer_namey,grocer_xall,grocer_namexos,grocer_prests,...
   grocer_boundsvarb,grocer_nonna]=...
      explouniv(grocer_namey,lstcat(grocer_comp,varargin),[],...
      ['endogenous';'exogenous'],%t,grocer_dropna)
   [grocer_namecomp,grocer_listtsc,grocer_vecc,grocer_indtsc,...
   grocer_indvecc,grocer_indctec,grocer_nz]=...
   explovars(grocer_comp,'exogenous')
   grocer_z=grocer_xall(:,1:grocer_nz)
   grocer_x=grocer_xall(:,grocer_nz+1:$)
else
   grocer_nz=0
   [grocer_y,grocer_namey,grocer_xall,grocer_namexos,grocer_prests,...
   grocer_boundsvarb,grocer_nonna]=explouniv(grocer_namey,varargin,...
   [],['endogenous';'exogenous'],%t,grocer_dropna)
   grocer_z=[]
   grocer_x=grocer_xall
end
 
if ~exists('grocer_estim1','local') then
   grocer_estim='ols'
   grocer_estim1=ols1a
   grocer_estim2=ols2a
   grocer_estim3=ols3a
end
 
if ~exists('win','local') then
   grocer_win=floor(5*size(grocer_y,1)^0.25)
end
 
[nobs,nx]=size(grocer_x)
nvar=nx+grocer_nz
[test_func,m2prt_test,nvarmax]=auto_test(grocer_names,grocer_ltest,nobs,nvar)
 
[nobs,nx]=size(grocer_x)
if nvar >= nvarmax then
   error('too many exogenous variables')
end
 
// if necessary set the significance levels of the test
strat_col=1+(nobs>60)+(nobs>120)+(nobs>200)+(nobs>1000)
if grocer_strategy == 'liberal' then
   levels=grocer_libtab(:,strat_col)
else
   levels=grocer_constab(:,strat_col)
end
 
if grocer_expert then
   grocer_strategy='expert'
end
if ~exists('grocer_gam','local') then
   grocer_gam=levels(1)
end
if ~exists('grocer_alpha','local') then
   grocer_alpha=levels(2)
end
if ~exists('grocer_f0_tdo','local') then
   grocer_f0_tdo=levels(3)
end
if ~exists('grocer_f0_bup','local') then
   grocer_f0_bup=levels(4)
end
 
indcte=search_cte([grocer_z grocer_x])
prescte=~isempty(indcte)
 
// define basic results tlists and update indcte
select grocer_estim
case 'ols'
   lrmod=def_results(grocer_y,grocer_namey,nobs,grocer_prests,prescte,...
          indcte,grocer_boundsvarb,m2prt_test,grocer_dropna,grocer_nonna)
 
case 'nwest'
   lrmod=def_results(grocer_y,grocer_namey,nobs,grocer_prests,prescte,...
          indcte,grocer_boundsvarb,m2prt_test,grocer_dropna,grocer_nonna,grocer_win)
 
end
// estimate the initial model
r1_00=grocer_estim2(grocer_y,[grocer_z grocer_x],prescte,lrmod(2))
r1_00('namex')=grocer_namexos
[val,p]=test_func(r1_00)
r1_00('spec_test')=[val p]
 
if ~exists('grocer_eta','local') then
   grocer_eta=0.01*ones(size(p,1),1)
else
   grocer_eta=vec2col(grocer_eta)
end
 
for i=1:size(p,1)
   if p(i) < grocer_eta(i) then
// must adjust the size level of the corresponding
// specification test
      if p(i) > %eps*grocer_descent then
         grocer_eta(i)=p(i)/grocer_descent
      else
// the adjustment would lead to a level lower than
// scilab precision ; so set it to
         grocer_eta(i)=0
      end
      disp(' ')
      chars='!!!!!!!!!!'
      disp(chars+chars+chars+chars+chars+chars)
      warning('level of specification test '+ m2prt_test(i+1)+' had ...
           to be ajusted to:'+string(grocer_eta(i)))
      disp('a) this can be the sign of a missing variable in your model')
      disp('b) it could be worthwhile to check what happens if you run the ')
      disp('programm without the corresponding specification test')
      disp(chars+chars+chars+chars+chars+chars)
      disp(' ')
   end
end
 
results=tlist(['results';'meth';'estim';'strategy';'alpha';'f0_tdo';'f0_bup';...
         'gam';'eta';'comp';'criterion';'f_test';'m_test';...
         'initial model';'ending reason';'final model'...
          ],'automatic',grocer_estim,grocer_strategy,grocer_alpha,grocer_f0_tdo,...
          grocer_f0_bup,grocer_gam,grocer_eta,grocer_z,grocer_crit,test_func,m2prt_test,r1_00,[],[])
 
if grocer_estim == 'nwest' then
   results(1)(1+$)='wind length'
   results('wind length')=grocer_win
end
 
// stage 0: eliminate variables at a loose significance level
// and estimate the bottom-up model
 
[done,results,indpvalue1_0] = auto_stage0(grocer_y,grocer_x,...
                            indcte,r1_00,lrmod(1),grocer_alpha,...
                            grocer_f0_tdo,grocer_eta,grocer_z,grocer_f0_bup,results)
if ~done then
   r1_0=results('stage 0 model')
   [gpvalue,indpvalue]=gsort(r1_0('pvalue')(grocer_nz+1:$),'g','d')
   xpath=grocer_x(:,indpvalue1_0)
   name1_0=grocer_namexos([[1:grocer_nz]' ; grocer_nz+indpvalue1_0])
   indcte1=find([[1:grocer_nz]' ; grocer_nz+indpvalue1_0] == indcte)
 
   listeq1=list()
   nonsignif=sum(gpvalue > grocer_alpha)
   if nonsignif == 0 then
      results('ending reason')='all variables significant in stage 0 model'
      results('final model')=r1_0
 
   elseif nonsignif == 1 & size(gpvalue,1) ==1 then
      results('ending reason')='final model is empty'
      results('final model')=[]
 
   else
// there is more than one variable left and it is non significant
      for i=1:nonsignif
         firstelim=indpvalue(i)
         listeq1=auto_stage1(grocer_y,xpath,listeq1,...
                indpvalue,indcte1,i,r1_0,lrmod(1),...
               grocer_alpha,grocer_eta,grocer_z)
      end
      sizelst1=length(listeq1)/3
      list1_2=list()
      for i=1:sizelst1
         if typeof(listeq1(3*i)) == 'results' then
            list1_2($+1)=r1_0('namex')(grocer_nz+listeq1(3*i-2))
            list1_2($+1)=listeq1(3*i-1)
            list1_2($+1)=listeq1(3*i)
         end
      end
 
      nbmodst1_0=length(list1_2)/3
      nbmodst1=length(listeq1)/3
      if size(grocer_groups_pval,1) ~= 0 then
         [results,r_checks,nbmodst1,list1_2]=...
         check_groups(results,grocer_y,grocer_z,grocer_groups_pval,...
         gpvalue,xpath,grocer_gam,grocer_eta,lrmod(1),list1_2,nbmodst1_0,indcte1)
      end
 
      if  nbmodst1_0 == 0 then
    // either: when you withdraw any variable, the model does not pass the specification tests
   // in that case listeq1 is empty
   // or: the final model is empty
   // in that case listeq1 has a path, but no model
 
         if isempty(listeq1) then
            results('ending reason')='specification tests hinder any ...
            reduction from stage 0 model'
            results('final model')=r1_0
 
         else
            results(1)($+1)='stage 1 paths'
            results('stage 1 paths')=list(listeq1(3*[1:nbmodst1]-2))
            if grocer_nz then
               rn=grocer_estim3(grocer_y,grocer_z,grocer_namexos,[1:grocer_nz]',prescte,...
                  lrmod(prescte+1))
               [val,p]=test_func(rn)
               results('ending reason')='final model contains only compulsory variables'
               rn('spec_test')=[val p]
               results('final model')=rn
            else
               results('ending reason')='final model is empty'
            end
         end
 
      else
         results(1)($+1)='stage 1 models'
         results('stage 1 models')=list1_2
         results(1)($+1)='stage 1 paths'
         results('stage 1 paths')=list(list1_2(3*[1:nbmodst1_0]-2))
 
         if nbmodst1 == 1 then
            results('ending reason')='only one model selected by ...
                       stage 1'
            results('final model')=list1_2(3)
         else
            numx3=fuslist(list(list1_2(3*[1:nbmodst1_0]-1)))
            prescte=or([[1:grocer_nz]' ; grocer_nz+numx3] == indcte1)
            r1_3=grocer_estim3(grocer_y,[grocer_z xpath],name1_0,[[1:grocer_nz]' ; grocer_nz+numx3],...
              prescte,lrmod(prescte+1))
            [val,p]=test_func(r1_3)
            r1_3('spec_test')=[val p]
 
            results(1)($+1)='stage 1 union model'
            results('stage 1 union model')=r1_3
            accept = ~zeros(nbmodst1_0,1)
            for i=1:nbmodst1_0
               if list1_2(3*i)('nvar') == r1_3('nvar') then
          // selected model = union model; cannot perform
          // wald test, but must accept selected model !
                  accept(i) = %t
               else
                  [fstat,fp]=waldf0(list1_2(3*i),r1_3)
                  accept(i) = (fp>grocer_gam)
               end
            end
            [accept,ordaccept] = gsort(bool2s(accept),'g','d')
            sumaccept=sum(accept)
            select sumaccept
 
 
// the following lines have been written to follow Hendry &
// Krozlig 1999 paper, but lead sometimes to unsatisfactory
//  results, so I have put them as commentary; feel free to
// add them again !
//      case 0 then
// all terminal models are rejected
//      results('ending reason')='all stage 1 models ...
//                       rejected against their union'
//      results('final model')=r1_3
 
            case 1 then
         //   only one terminal model is not rejected:
         //  it is the final model
               results('ending reason')='only one stage 1 model ...
                       not rejected against their union'
               results('final model')=list1_2(3*ordaccept(1))
            else
           // more than one model are accepted
           // form the union of the accepted models
               if sumaccept  == 0 then
                  indx2_00=numx3
               else
                  indx2_00=ordaccept(1:sumaccept)
               end
               listr2_00=list(list1_2(3*ordaccept-1))
               listx2_00=fuslist(listr2_00)
               indcte2=find([[1:grocer_nz]' ; grocer_nz+listx2_00] == indcte1)
               prescte2 = (~isempty(indcte2))
               x2_00=xpath(:,listx2_00)
               name2_00=name1_0([[1:grocer_nz]' ; grocer_nz+listx2_00])
               r2_00=grocer_estim3(grocer_y,[grocer_z xpath],name1_0,[[1:grocer_nz]' ; ...
               grocer_nz+listx2_00],prescte2,lrmod(prescte2+1))
               [val,p]=test_func(r2_00)
               r2_00('spec_test')=[val p]
               results(1)($+1)='stage 2.0 model'
               results($+1)=r2_00
 
               listeq2=list()
               [gpvalue2,indpvalue2_0]=gsort(r2_00('pvalue')(grocer_nz+1:$),'g','d')
               nonsignif=sum(gpvalue2 > grocer_alpha)
 
               for i=1:nonsignif
                  firstelim=indpvalue2_0(i)
                  listeq2=auto_stage1(grocer_y,x2_00,listeq2,...
                    indpvalue2_0,indcte2,i,r2_00,lrmod(1),...
                    grocer_alpha,grocer_eta,grocer_z)
               end
               sizelst2=length(listeq2)/3
               list2_2=list()
               for i=1:sizelst2
                  if typeof(listeq2(3*i)) == 'results' then
                     list2_2($+1)=r2_00('namex')(grocer_nz+listeq2(3*i-2)   )
                     list2_2($+1)=listeq2(3*i-1)
                     list2_2($+1)=listeq2(3*i)
                  end
               end
 
               results(1)($+1)='stage 2.1 models'
               results($+1)=list2_2
 
               nbmodst2=length(list2_2)/3
 
               select nbmodst2
 
               case 0 then
                  if nonsignif == 0 then
                     results('ending reason')='all variables significant in stage 2.0 model '
                  else
                     results('ending reason')='no variable can be withdrawn in stage 1 union model '
                  end
                  results('final model')=r2_00
 
               case 1 then
                  results('ending reason')='only one model selected by stage 2'
                  results('final model')=list2_2(3)
                  results(1)($+1)='stage 2 models'
                  results($+1)=list2_2
 
               else
                  numxd=fuslist(list(list2_2(3*[1:nbmodst2]-1)))
                  prescte=or(numxd == indcte2)
                  r2_3=grocer_estim3(grocer_y,[grocer_z x2_00],name2_00,...
                   [[1:grocer_nz]' ; grocer_nz+numxd],prescte,lrmod(prescte+1))
                  [val,p]=test_func(r2_3)
                  r2_3('spec_test')=[val p]
                  results(1)($+1)='stage 2 union model'
                  results('stage 2 union model')=r2_3
                  listr2_3=list(list2_2(3*[1:nbmodst2]))
                  accept=~zeros(nbmodst2,1)
                  for i=1:nbmodst2
                     if listr2_3(i)('nvar') == r2_3('nvar') then
                     // selected model = union model ; cannot perform
                     // wald test, but must accept selected model !
                        accept(i) = %t
                     else
                        [fstat,fp]=waldf0(listr2_3(i),r2_3)
                        accept(i) = (fp>grocer_gam)
                     end
                  end
                  [accept,ordaccept] = gsort(bool2s(accept),'g','d')
                  sumaccept=sum(accept)
 
                  select sumaccept
                  case 0 then
                 // all terminal models are rejected
                     results('ending reason')='all stage 2 models rejected ...
                          against their union'
                     results(1)($+1)='stage 2 models'
                     [val,p]=test_func(r2_3)
                     results($+1)=list([],[],r2_3,[val p])
                     results('final model')=r2_3
 
                  case 1 then
                 // only one terminal model is not rejected:
                 // it is the final model
                     results('ending reason')='only one stage 2 model not ...
                              rejected against their union'
                     results('final model')=list2_2(3*ordaccept(1))
//list(list2_2(3*ordaccept(1)-2),...
//                               list2_2(3*ordaccept(1)-1),...
//                               list2_2(3*ordaccept(1)))
                  else
                 // more than one model are accepted
                     results(1)($+1)='stage 2 models'
                     crit0=%inf
                     lst2_mod=list()
                     for i=1:sumaccept
                        r=list2_2(3*ordaccept(i))
                        crit=r(grocer_crit)
                        if crit < crit0 then
                           indmof=i
                           crit0=crit
                        end
                        lst2_mod($+1)=list2_2(3*ordaccept(i)-2)
                        lst2_mod($+1)=list2_2(3*ordaccept(i)-1)
                        lst2_mod($+1)=list2_2(3*ordaccept(i))
                     end
                     results('stage 2 models')=lst2_mod
                     results('ending reason')='stage 2 models selected by '...
                        +grocer_crit+' criterion'
                     results('final model')=list2_2(3*ordaccept(indmof))
                  end
               end
            end
         end
      end
   end
end
// reliability of the selected variables
if results('ending reason') ~= 'final model is empty' then
   resl=reliability(results('final model'),grocer_wreliab,grocer_alpha,'noprint')
else
   resl=[]
end
results(1)($+1)='reliab'
results('reliab')=resl
 
// print the results
prtauto(results,grocer_lprt,%io(2))
 
endfunction
