function [results]=nwest2a(y,x,prescte,results)
 
// PURPOSE: One of the numerous functions performing ordinary
// least squares: this one assumes that x et y are already a
// matrix and a vector respectively, that the existence or
// absence of a constant has already been determined (and
// stored in a boolean) and that a result tlist exists which
// can be filled.
// ------------------------------------------------------------
// INPUT:
// * y = dependent variable vector (nobs x 1)
// * x = independent variables matrix (nobs x nvar)
// * prescte = a boolean indicating whether the model contains
//   a cte term or not
// * results = a tlist containing all necessary fields, with
//   the fields 'meth','nobs','y','namey','prests','ym' are
//   already filled
// ------------------------------------------------------------
// RETURNS: a tlist
// results = a tlist with
//   . results('meth')  = 'ols'
//   . results('y')     = y data vector
//   . results('x')     = x data matrix
//   . results('nobs')  = nobs
//   . results('nvar')  = nvars
//   . results('beta')  = bhat
//   . results('yhat')  = yhat
//   . results('resid') = residuals
//   . results('vcovar') = estimated variance-covariance matrix of
//     beta
//   . results('sige')  = estimated variance of the residuals
//   . results('sige')  = estimated variance of the residuals
//   . results('ser')  = standard error of the regression
//   . results('tstat') = t-stats
//   . results('pvalue') = pvalue of the betas
//   . results('dw')    = Durbin-Watson Statistic
//   . results('condindex') = multicolinearity cond index
//   . results('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . results('rsqr')  = rsquared
//   . results('rbar')  = rbar-squared
//   . results('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . results('pvaluef') = its significance level
//---------------------------------------------------
// Copyright: Eric Dubois 2009
// http://dubois.ensae.net/grocer.html
 
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
 
 
[nobs,nvar]=size(x)
 
[bet,xpxi]=ols0(y,x)
yhat = x*bet
resid = y-yhat
sigu =resid'*resid
sige= sigu/(nobs-nvar)
//ser=sqrt(sige)
vcovar=sige*xpxi
 
emat=ones(nvar,1) .*. resid'
 
hhat=emat .* x'
xuux=hhat(:,1:nobs)*hhat(:,1:nobs)'
 
win=results('win')
for i=1:win
   zi=hhat(:,(i+1):nobs)*hhat(:,1:nobs-i)'
   gi=zi+zi'
   xuux=xuux+(1-i/(win+1))*gi
end
 
xpxia = xpxi*xuux*xpxi;
tmp = sqrt(diag(xpxia));
tstat=bet ./ tmp
 
ediff = resid(2:nobs)-resid(1:nobs-1)
dw = ediff'*ediff/sigu
// durbin-watson
 
df=nobs-nvar
pvalue=ones(nvar,1)
for i=1:nvar
   pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2
end
 
//condindex=bkwols(x)
 
results('x') = x
results('nvar')  = nvar
results('beta')  = bet
results('yhat')  = yhat
results('resid') = resid
results('sige')  = sige
results('sigu')  = sigu
results('tstat') = tstat
results('pvalue') = pvalue
results('dw') = dw
results('ser') = sqrt(sige)
results('vcovar') = xpxia
results('condindex') = bkwols(x)
results('prescte') = prescte
 
if prescte & nvar ~=1 then
// there is a constant and at least another exogenous variable:
// R and Rbar make sense
  rsqr1 = sigu;
  rsqr2 = results('ym')'*results('ym');
  // r-squared
  rsqr=1-rsqr1/rsqr2
  results('rsqr') =rsqr
  nobsm1=nobs-1
  nvarm1=nvar-1
  results('rbar') = 1-rsqr1/df/rsqr2*nobsm1
  // rbar-squared
  f=rsqr/(1-rsqr)*df/(nvar-1)
  results('f') = f
  results('pvaluef') = 1-cdff("PQ",f,nvarm1,nobs-nvar)
end
 
results('aic')=log(results('sigu')/nobs)+results('nvar')*2/results('nobs')
results('bic')=log(results('sigu')/nobs)+results('nvar')*log(results('nobs'))/results('nobs')
results('hq')=log(results('sigu')/nobs)+results('nvar')*2*log(log(results('nobs')))/results('nobs')
 
endfunction
