  function rolsecm = olsecm(grocer_p,grocer_namey0,varargin)
 
// PURPOSE: Error correction test for cointegration
// ------------------------------------------------------------
// REFERENCE: N. R. Ericsson & J. G. MacKinnon (2002),
// "Distribution of error correction test for cointegration"
// Econometrics Journal, 5, 285-318
// ------------------------------------------------------------
// INPUT:
// * grocer_p = order of time polynomial in the error correction part
//  . -1, no deterministic part (default)
//  .  0, for constant term
//  .  1, for constant plus time-trend
//  .  2, for constant plus quadratic time-trend
// * grocer_namey = the exogenous variable wich can be
//   a time series, a real (nx1) vector or a
//   string equal to the name of a time series or a (nx1) real
//   vector between quotes
//   WARNING: if the user does not provide a cointegrating vector
//    this serie should be in level because it is also
//    used for the estimation of error correction part
// * varargin = arguments which can be:
//   . 'coint=[var1 var2 ... varn]'
//      or 'coint=''y+a1*var1+a2*var2+...+an*varn'''
//          in case of constrained or previously estimated
//          cointegrating relation
//      with vari: the ith exogenous variable
//      in the cointegrating relation
//   . the exogenous variables of the ECM which can be
//      - a time series
//      - a real (nxp) vector
//      - a string equal to the name of a time series or a (nxp)
//        real vector between quotes
//   . the string 'noprint' if the user doesn't want the
//     to print the results of the regression
//   . the string 'dropna' if the user wants to delete NAs
//     (this option should be used when dealing with daily and weekly TS)
// ------------------------------------------------------------
// OUTPUT:
// rolsecm = a results tlist with
//   . rolsecm('meth')  = 'olsecm'
//   . rolsecm('y')     = y data vector of the ECM
//   . rolsecm('x')     = x data matrix of the ECM
//   . rolsecm('ecm')     = data matrix of the error correction
//   . rolsecm('nobs')  = # observations in the ECM
//   . rolsecm('nvar')  = # variables in the ECM
//   . rolsecm('beta')  = bhat
//   . rolsecm('yhat')  = yhat
//   . rolsecm('resid') = residuals of the auxiliary regression
//   . rolsecm('vcovar') = estimated variance-covariance matrix of
//     beta
//   . rolsecm('sige')  = estimated variance of the residuals
//   . rolsecm('sigu')  = sum of squared residuals
//   . rolsecm('ser')  = standard error of the regression
//   . rolsecm('tstat') = t-stats
//   . rolsecm('pvalue') = pvalue of the betas
//   . rolsecm('dw')    = Durbin-Watson Statistic
//   . rolsecm('condindex') = multicolinearity cond index
//   . rolsecm('prescte') = boolean indicating the absence of a
//     constant in the regression
//   . rolsecm('test p-value') = the (approximate) p-value of
//     the test
//   . rolsecm('test crit. value') = the Ericsson-MacKinnon
//     critical value of the test
//   . rolsecm('deterministic') = type of deterministic part
//     in the cointegrating vector
//   . rolsecm('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rolsecm('namey') = name of the y variable of the ECM
//   . rolsecm('namex') = name of the x variables of the ECM
//   . rolsecm('name ecm') = name of the variables of the
//      error correction part
//   . rolsecm('bounds') = if there is a timeseries in the
//     regression, the bounds of the regression
//   . rolsecm('like') = log-likelihood of the regression
//   . rolsecm('dropna') = boolean indicating if NAs had
//		    been dropped
//   . rolsecm('nonna') = vector indicating position of
//     non-NA values (if the option 'dropna' was active)
// ------------------------------------------------------------
// Copyright: Emmanuel Michaux 2007
// http://dubois.ensae.net/grocer.html
 
 
 
out = %io(2);
 
grocer_prt=%t;
grocer_dropna=%f;
grocer_coint = [];
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      grocer_st=strsubst(varargin(grocer_i),' ','');
      if part(grocer_st,1:5) == 'coint' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null();
      elseif grocer_st == 'noprint' then
         grocer_prt=%f;
         varargin(grocer_i)=null();
      elseif grocer_st == 'dropna' then
         grocer_dropna = %t;
         varargin(grocer_i)=null();
      end
   else
      error(typeof(varargin(grocer_i))+': not a valid type in olsecm');
   end
end
 
if grocer_p>2 then
  error('critical values are only available up through quadratic time-trend');
end
 
// test the format of cointegrating vector
if length(grocer_coint) == [] then
  error('cointegrating variables are missing');
else
  cci=0;
  if max(size(grocer_coint)) == 1 then
    fs = strindex(grocer_coint,['+','-']);
    if length(fs)>0 then
      cci = 1;
    else
      error('wrong specification of the cointegrating vector');
    end
  end
end
 
// if the cointegrating has to be estimated
grocer_exo = varargin
if ~cci then
 
  for i=1:max(size(grocer_coint))
    grocer_exo($+1)= grocer_coint(i)
  end
 
  [grocer_ly,grocer_namely,grocer_z,grocer_namez,grocer_prests,grocer_boundsvarb,nonna]=...
      explouniv(grocer_namey0,grocer_exo,[],['endogenous';'exogenous'],%t,grocer_dropna);
 
  // exogenous varaible that only enters the ECM
  grocer_x = grocer_z(2:$,1:length(varargin)); // delete first obs. because ECM is estimated in diff
  grocer_namexos = grocer_namez(1:length(varargin));
  grocer_z(:,1:length(varargin)) = [];
  grocer_namez(1:length(varargin)) = [];
 
  // constructs the determnistic part
  if grocer_p >= 0 then
    n=size(grocer_ly,1);
    grocer_z=[grocer_z ptrend(grocer_p,n)];
    grocer_namez=[grocer_namez ; 'cte'];
    if grocer_p > 0 then
      grocer_namez=[grocer_namez ; 't' ];
    end
    if grocer_p > 1 then
      grocer_namez=[grocer_namez ; 't^'+string([2:grocer_p])];
    end
  end
 
  // estimate the cointegrating vector
  rols=ols2(grocer_ly,grocer_z);
 
  // saves the names, the bounds if the regression involves ts
  rols(1)($+1) = 'prests';
  rols(1)($+1) = 'namex';
  rols(1)($+1) = 'namey';
  rols(1)($+1) = 'dropna';
  rols('prests')=grocer_prests;
  rols('namex')=grocer_namez;
  rols('namey')=grocer_namely
  rols('dropna')=grocer_dropna;
 
  if grocer_prests then
    rols(1)($+1) = 'bounds';
    rols('bounds')=grocer_boundsvarb;
  end
 
  if grocer_dropna then
     rols(1)($+1)='nonna';
     rols('nonna')=nonna;
  end
 
  // print the results if necessary
  if grocer_prt then
    write(out,'   First stage: estimation of the cointegrating vector');
    write(out,'');
    prtuniv(rols);
  end
 
else
    grocer_civ = evstr(grocer_coint);// evaluates coinegrating vector provided by the user
 
    // take one lag
    if typeof(grocer_civ)== "ts" then
      grocer_civ = lagts(grocer_civ);
    else
      grocer_civ = grocer_civ(1:$-1,:);
    end
 
    grocer_exo($+1) = 'grocer_civ';
    [grocer_y,grocer_namey,grocer_x,grocer_namexos,grocer_prests,grocer_boundsvarb,nonna]=...
      explouniv(grocer_namey0,grocer_exo,[],['endogenous';'exogenous'],%t,grocer_dropna);
 
  // print the results if necessary
  if grocer_prt then
    write(out,'   First stage: cointegrating vector provided by the user');
    write(out,'');
    write(out,grocer_coint);
    write(out,'');
  end
end
 
// estimation of the ECM
if grocer_prests then
 
  // if the cointegrating has to be estimated
  if ~cci then
    bo=rols('bounds');
    nvci = rols('nvar')-grocer_p; // # of var, appart the determnistic part, in the coinetgrating vector
    grocer_boundsvarb=lagbounds(grocer_boundsvarb,1);     // lags bounds by one period: cointegrating vector en enters MCE with 1 lag
    grocer_namey = 'delts('+grocer_namely+')'; // first diff grocer_namely = exogenous varaible of the ECM
    grocer_y  = trimr(tdiff(grocer_ly,1),1,0);
    grocer_x = [grocer_x,rols('resid')(1:$-1)]
    grocer_namexos($+1) = 'lag of coint. vec.';
  else
    stci=strindex(grocer_coint,['+','-']);
    nvci=length(stci)+1-grocer_p-1; // # of var, appart the determnistic part, in the coinetgrating vector
    grocer_namexos($) = 'lagts('+grocer_coint+')';
  end
 
else
 
  if ~cci then
    grocer_y = trimr(tdiff(grocer_ly,1),1,0); // first diff grocer_namely = exogenous varaible of the ECM
    grocer_civ = trimr(lag(rols('resid')),1,0);// keeps cointegrating vector
    nvci = rols('nvar')-grocer_p;// # of var, appart the determnistic part, in the coinetgrating vector
 
    ny = size(grocer_y,1);
    nx = size(grocer_x,1);
 
    if nx ~=ny then
      error('x and y must have same # obs');
    end
 
    grocer_x =[grocer_x,grocer_civ]
    // computes names
    grocer_namey = 'del('+grocer_namey0+')';
    grocer_namexos($+1) = 'lag of coint. vec.';
 
  else
    stci=strindex(grocer_coint,['+','-']);
    nvci=length(stci)+1-grocer_p-1;// # of var, appart the determnistic part, in the coinetgrating vector
    grocer_namexos($) = 'lag('+grocer_coint+')';
  end
end
 
rolsecm= ols2(grocer_y,grocer_x);
rolsecm('meth') = 'olsecm';
 
// computes Ericsson-MacKinnon critical values and p-values
[crit,pval] = critecm(rolsecm('tstat')($),nvci,grocer_p,rolsecm('nobs'),rolsecm('nvar'));
 
// saves the names, the bounds if the regression involves ts
rolsecm(1)($+1) = 'prests';
rolsecm(1)($+1) = 'namex';
rolsecm(1)($+1) = 'namey';
rolsecm(1)($+1) = 'name ecm';
rolsecm(1)($+1) = 'dropna';
rolsecm(1)($+1) = 'test p-value'
rolsecm(1)($+1) = 'test crit. value'
rolsecm(1)($+1) = 'deterministic'
rolsecm('prests')=grocer_prests;
rolsecm('namex')=grocer_namexos;
rolsecm('namey')=grocer_namey;
if cci then
  rolsecm('name ecm') = grocer_coint
else
  rolsecm('name ecm') = grocer_namez
end
rolsecm('dropna')=grocer_dropna;
rolsecm('test p-value') = pval
rolsecm('test crit. value') =crit
rolsecm('deterministic') =grocer_p
if grocer_prests then
   rolsecm(1)($+1) = 'bounds';
   rolsecm('bounds')=grocer_boundsvarb;
end
 
if grocer_dropna then
   rolsecm(1)($+1)='nonna';
   rolsecm('nonna')=nonna;
end
 
if grocer_prt then
  write(out,'   Second stage: estimation of the ECM');
  write(out,'');
  prtuniv(rolsecm);
  prtunitr(rolsecm);
end
 
endfunction
 
