function [result]=johansen(grocer_k,varargin)
 
// PURPOSE: perform Johansen cointegration tests
// ------------------------------------------------------------
// * References:
// - Johansen (1988), 'Statistical Analysis of
// Co-integration vectors', Journal of Economic Dynamics and
// Control, 12, pp. 231-254.
// - Jueslius (2006): The Cointegrated VAR Model: Methodlogy
// and Applications, Oxford University Press.
// ------------------------------------------------------------
// INPUT:
// * k = number of lagged difference terms used when
//                 computing the estimator
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector
//   . a string equal to the names of a time series or a (nx1)
//     real vector between quotes
//   . the string 'exo_st=exo_st1,exo_st2,...exo_stn' where
//     exo_sti is the names of an exogenous variable to be added
//     to the short run dynamic of the VAR
//   . the string 'exo_lt=exo_lt1,exo_lt2,...exo_ltn' where
//     exo_lti is the names of an exogenous variable to be added
//     to the cointegrating vectors
//   . the string 'NBoot=n' where n is the number of bootstrap
//     draws (default: 999)
//   . the string 'max_nonzeros=n' where n is the maximum
//     number of zeros a variable must have to be considered
//     as a dummy (default: 4)
//   . the string 'noprint' if the user doesn't want the
//     to print the results of the regression
//  - 'dropna' if the user wants to remove the NA values
//     from the data
// ------------------------------------------------------------
// OUTPUT:
// result = a results tlist with:
//   - result('meth') = 'johansen'
//   - result('namey') = the names of the variables (m x 1)
//   - result('y') = matrix of values for the variables
//                    (m x 1)
//   - result('namexo_lt') = the names of the exogenous variables
//     in the cointegrating vectors
//   - result('exo_lt') = the matrix of the exogenous variables
//     in the cointegrating vectors
//   - result('namexo_st') = the names of the exogenous variables
//     in the short run dynamic of the VAR
//   - result('exo_st') = the matrix of the exogenous variables
//     in the short run dynamic of the VAR
//   - result('dy') = the matrix of the differentiated endogenous
//     variables
//   - result('exo') = the matrix of the variables in the short
//     run dynamics (lagged diffrentiated endogenous variables +
//     short run exogenous variables)
//   - result('lagy') = the matrix of the variables in the
//     cointegrating relations (lagged endogenous variables +
//     long run exogenous variables)
//   - result('nobs') = # of observations
//   - result('nvar') = # of variables
//   - result('nlags') = # of lags of the VAR
//   - result('eig') = eigenvalues (m x 1)
//   - result('evec') = eigenvectors (m x m)
//   - result('pi') = coefficients of the short run dynamic
//   - result('lr1') = likelihood ratio trace statistics for
//                       r=0 to m-1, a (m x 1) vector
//   - result('lr2') = maximum eigenvalue statistic for r=0
//                       to m-1, a (m x 1) vector
//   - result('dropna') = boolean indicating if NAs have
//     been dropped
//   - result('nonna') = vector indicating position of
//     non-NA values (if the option 'dropna' was active)
//   - result('max non zeros') = maximum number of zeros a
//     variable had to be considered as a dummy
//   - result('NBoot') = # of bootstrap draws
//   - result('alpha') = value of the error correction
//     coefficients
//   - result('cvt') = critical values for trace statistic
//                        (m x 3) vector [90% 95% 99%]
//   - result('cvm') = critical values for max eigen value
//                       statistic (3 x m) vector [90% 95% 99%]
//   - result('p trace') = p-value for the trace statistic
//     calculated with the standard bootstrap method
//   - result('p lmax') = p-value for the lambda-max statistic
//     calculated with the standard bootstrap method
//   - result('p double trace') = p-value for the trace statistic
//     calculated with the double bootstrap method
//   - result('p double lmax') = p-value for the lambda-max
//     statistic calculated with the double bootstrap method
//   - result('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   - result('bounds') = if there is a time series in the
//     regression, the bounds of the regression
// ------------------------------------------------------------
// NOTE: uses the function johansen_eigen adapted from a
// programm by James LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the trace and max eignevalues statistics
// ------------------------------------------------------------
// Copyright: Eric Dubois 2009
// http://dubois.ensae.net/grocer.html
 
 
// set defaults
 
 
grocer_max_nonzeros=4
grocer_dropna=%f
grocer_prt=%t
grocer_exo_lt=[]
grocer_exo_st=[]
grocer_NBoot=999
 
// separates from the list of variable arguments the list of
// exogenous variables and 'noprint' if the user has given this
// argument
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   grocer_argi=varargin(grocer_i)
   if typeof(grocer_argi) == 'string' then
      grocer_argin=strsubst(grocer_argi,' ','')
      grocer_str7=part(strsubst(grocer_argin,' ',''),1:7)
 
      if or(grocer_str7 == ['exo_lt=';'exo_st=']) then
         execstr('grocer_'+grocer_str7+'string2vec(part(grocer_argi,8:length(grocer_argi)),'';'')')
         varargin(grocer_i) =null()
      elseif part(grocer_argin,1:6) == 'NBoot=' then
         execstr('grocer_'+grocer_argin)
         varargin(grocer_i) =null()
      elseif grocer_argin == 'dropna' then
         grocer_dropna=%t
         varargin(grocer_i) =null()
      elseif grocer_argin == 'noprint' then
         grocer_prt=%f
         varargin(grocer_i) =null()
      end
   end
end
 
if exists('grocer_boundsvar') then
   // get the bounds k+1 times back, because the y variable in johansen
   // is lead k+1 times
   // transfer the boundsvar variable in the function
   // so that it can be transformed in the function
   if size(grocer_boundsvar,1) > 2 then
      error('you cannot use discountinous bounds in johansen')
   end
   if grocer_boundsvar ~= [] then
      grocer_boundsvar=[num2date(date2num(grocer_boundsvar(1))-grocer_k-1,...
                   date2fq(grocer_boundsvar(1))) ; grocer_boundsvar(2)]
   end
end
 
[mats,names,prests,b,nonna]=explon(list(varargin,grocer_exo_lt,grocer_exo_st),...
['endogenous' 'l.t. exogenous' 's.t. exogenous'],[],%t,grocer_dropna)
 
y=mats(1)
exo_lt=mats(2)
exo_st=mats(3)
 
// recover the names of the variables
namey=names(1)
namexo_lt=names(2)
namexo_st=names(3)
 
dy = tdiff(y,1);
exo=trimr([mlag(dy,grocer_k) exo_st],grocer_k+1,0)
lagy = trimr(lag([y exo_lt],1),grocer_k+1,0)
dy=trimr(dy,grocer_k+1,0)
[nobs,ny]=size(dy)
nlagy=size(lagy,2)
 
[flag,a,evec,lr1,lr2,pi]=johansen_eigen(dy,exo,lagy)
// set up first results tlist
result = tlist(['results';'meth';'namey';'y';'namexo_lt';'exo_lt';...
'namexo_st';'exo_st';'dy';'exo';'lagy';'nobs';'nvar';...
'nlags';'eig';'evec';'pi';'lr1';'lr2';'dropna';'max non zeros';...
'NBoot']...
,'johansen',namey,y,namexo_lt,exo_lt,namexo_st,exo_st,dy,exo,lagy,...
nobs,ny,grocer_k,a,evec,pi,lr1,lr2,grocer_dropna,grocer_max_nonzeros,grocer_NBoot)
 
// calculate unrestricted residuals
b=ols0(dy,[lagy*evec exo])
alpha=b(1:ny,:)
gam=b(ny+1:$,:)
 
resid=(dy-lagy*evec*alpha-exo*gam)*sqrt(nobs/(nobs-size(b,1)))
result(1)($+1)='alpha'
result('alpha')=alpha
 
resid=suppr_resid_dummies(resid,exo(:,ny*grocer_k+1:$),grocer_max_nonzeros)
nbresid=size(resid,1)
// do (Nboot*2 x nobs) draws in a unifrom law (the factor 2 because of the double boostrap
draws=floor(grand(grocer_NBoot*2,nobs,'unf',1,nbresid+1))
yn=y;
lt=[lagy ; zeros(1,nlagy)];
exon=[exo ; zeros(1,size(exo,2))];
lr1_boot=[zeros(grocer_NBoot,ny) ; lr1'];
lr2_boot=[zeros(grocer_NBoot,ny) ; lr2'];
lr1_boot2=[zeros(grocer_NBoot,ny) ; lr1'];
lr2_boot2=[zeros(grocer_NBoot,ny) ; lr2'];
lr1_p=zeros(ny,1)
lr2_p=zeros(ny,1)
lr1_p2=zeros(ny,1)
lr2_p2=zeros(ny,1)
 
dyn=zeros(nobs,ny)
nexo_lt=size(exo_lt,2)
 
for i=1:ny
   b=ols0(dy,[lagy*evec(:,1:i-1) exo])
   alpha=b(1:i-1,:)
   gam=b(i:$,:)
   drawi=0
 
   while drawi < grocer_NBoot
      drawi=drawi+1
      residn=resid(draws(drawi,:),:)
      for t=1:nobs
         dyn(t,:)=lt(t,:)*evec(:,1:i-1)*alpha+exon(t,:)*gam+residn(t,:) ;
         lt(t+1,1:ny)=lt(t,1:ny)+dyn(t,:);
         exon(t+1,1:ny*grocer_k)=[dyn(t,:) exon(t,1:ny*(grocer_k-1))];
      end
      [flag,an,evecn,lr1n,lr2n,pin]=johansen_eigen(dyn,exon(1:$-1,:),lt(1:$-1,:))
      if flag == 'not OK' then
         draws(drawi,:)=floor(grand(1,nobs,'unf',1,nbresid+1))
         drawi=drawi-1
      else
         lr1_boot(drawi,i)=lr1n(i)
         lr2_boot(drawi,i)=lr2n(i)
         b2=ols0(dyn,[lt(1:$-1,:)*evecn exon(1:$-1,:)])
         alpha2=b2(1:ny,:)
         gam2=b2(ny+1:$,:)
         residn_b=(dyn-lt(1:$-1,:)*evecn*alpha2-exon(1:$-1,:)*gam2)*sqrt(nobs/(nobs-size(b,1)))
         residn_fb=residn_b(draws(grocer_NBoot+drawi,:),:)
 
         b2c=ols0(dyn,[lt(1:$-1,:)*evecn(:,1:i-1) exon(1:$-1,:)])
         alpha2c=b2c(1:i-1,:)
         gam2c=b2c(i:$,:)
 
         for t=1:nobs
            dyn_fb(t,:)=lt(t,:)*evecn(:,1:i-1)*alpha2c+exon(t,:)*gam2c+residn_fb(t,:) ;
            lt(t+1,1:ny)=lt(t,1:ny)+dyn_fb(t,:);
            exon(t+1,1:ny*grocer_k)=[dyn_fb(t,:) exon(t,1:ny*(grocer_k-1))];
         end
         [flag,a_fb,evec_fb,lr1_fb,lr2_fb,pi_fb]=johansen_eigen(dyn_fb,exon(1:$-1,:),lt(1:$-1,:))
         if flag == 'not OK' then
            draws(grocer_NBoot+drawi,:)=floor(grand(1,nobs,'unf',1,nbresid+1))
            drawi=drawi-1
         else
            lr1_boot2(drawi,i)=lr1_fb(i)
            lr2_boot2(drawi,i)=lr2_fb(i)
         end
      end
   end
 
   lr1_boot(:,i)=gsort(lr1_boot(:,i),'g','i')
   lr1_p(i)=sum(lr1_boot(:,i) > lr1(i))/grocer_NBoot
   lr1_boot2(:,i)=gsort(lr1_boot2(:,i),'g','i')
   Q_B=lr1_boot2(max(floor(grocer_NBoot-lr1_p(i)*grocer_NBoot),1),i)
   lr1_p2(i)=sum(lr1_boot(:,i) > Q_B)/grocer_NBoot
 
   lr2_boot(:,i)=gsort(lr2_boot(:,i),'g','i')
   lr2_p(i)=sum(lr2_boot(:,i) > lr2(i))/grocer_NBoot
   lr2_boot2(:,i)=gsort(lr2_boot2(:,i),'g','i')
   Q_B=lr2_boot2(max(floor(grocer_NBoot-lr2_p(i)*grocer_NBoot),1),i)
   lr2_p2(i)=sum(lr2_boot(:,i) > Q_B)/grocer_NBoot
 
end
 
result(1)($+1)='cvt'
result(1)($+1)='cvm'
result(1)($+1)='p trace'
result(1)($+1)='p lmax'
result(1)($+1)='p double trace'
result(1)($+1)='p double lmax'
result('cvt')=lr1_boot(int([0.9 0.95 0.99]*grocer_NBoot),:)
result('cvm')=lr2_boot(int([0.9 0.95 0.99]*grocer_NBoot),:)
result('p trace')=lr1_p
result('p lmax')=lr1_p
result('p double trace')=lr1_p2
result('p double lmax')=lr2_p2
 
result(1)($+1) = 'prests'
result('prests')=prests
if prests then
   result(1)($+1) = 'bounds'
   result('bounds')=b
end
 
if grocer_dropna then
   result(1)($+1)='nonna'
   result('nonna')=nonna
end
 
if grocer_prt then
   prtjohan(result)
end
 
endfunction
