function [rkalman]=kalman(grocer_func,grocer_y,grocer_x,grocer_z,grocer_F,grocer_param,varargin)
 
// PURPOSE: maximum likelihood estimation of a kalman model:
//          y(t) = X(t)*B(t) + Z(t)*A + e(t), e(t) = N(0,R)
//          B(t) = Z(t) * B(t-1) + v(t),    v(t) = N(0,Q)
// ------------------------------------------------------------
// INPUT:
// * grocer_func = the function which transforms the parameters
//        into the matrix of variances (Q and R)
// * grocer_y = (nobs x 1) dependent variable vector
// * grocer_x = (nobs x 1) explanatory variable matrix
// * grocer_z = (nxl) data matrix of exogenous variables (or []
//    if there are no exogenous variables in the model)
// * grocer_F = the transfer matrix
// * grocer_param = a vector of parameters (sqrt of variances)
// * varargin = optional arguments which can be:
//   - 'priorb0=x' where x is (k x 1) vector with prior b0 values
//              (default = zeros(k,1), diffuse)
//   - 'priorv0=x' where x = (k x k) matrix with prior variance
//            for Q (default = eye(k)*1e+5, a diffuse prior)
//   - 'meth=x' where x is either 'maxlik' (default) or 'optim'
//     according to the optimization programm used
//   - any option to maxlik (see maxlik() for a list)
// ------------------------------------------------------------
// OUTPUT:
// rkalman = a results tlist with
//   - rkalman('meth') = 'kalman'
//   - rkalman('Q') = estimated Q
//   - rkalman('R') = estimated R
//   - rkalman('priorb0') = B(0/0)
//   - rkalman('priorv0') = sigma(0/0)
//   - rkalman('betat') = B(t/t)
//   - rkalman('betaf') = B(t/t-1)
//   - rkalman('betas') = B(t/T)
//   - rkalman('sigmatt') = sigma(t/t)
//   - rkalman('sigmatf') = sigma(t/t-1)
//   - rkalman('sigmats') = sigma(t/T)
//   - rkalman('param') = estimated parameters
//   - rkalman('vcov') = variance-covariance matrix of
//     estimated parameters
//   - rkalman('tstat') = Student's t of estimated parameters
//   - rkalman('y') = y
//   - rkalman('x') = x
//   - rkalman('z') = z
//   - rkalman('yhat') = X(t)*B(t)
//   - rkalman('resid') = y-X*B(t)
//   - rkalman('like') = log-likelihood
//   - rkalman('nobs') = # of observations
//   - rkalman('nvar') = # of exogenous variables
// ------------------------------------------------------------
// Copyright Eric Dubois 2002
 
 
 
 
[grocer_n,grocer_k] = size(grocer_x);
grocer_param=vec2col(grocer_param)
grocer_begsmooth=1
grocer_kalman_ik=eye(grocer_k,grocer_k)
grocer_priorb0 = zeros(grocer_k,1);
grocer_priorv0 = grocer_kalman_ik*100000;
grocer_meth = 'optim'
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   select type(varargin(grocer_i))
   case 10 then
      if part(varargin(grocer_i),[1:5]) == 'meth=' ...
      | part(varargin(grocer_i),[1:8]) == 'priorb0=' ...
      | part(varargin(grocer_i),[1:8]) == 'priorv0=' ...
      | part(varargin(grocer_i),[1:10]) == 'begsmooth=' ...
      then
         execstr("grocer_"+varargin(grocer_i))
         varargin(grocer_i)=null()
      end
   else
      disp(varargin(grocer_i))
      error('wrong type for entry # '+string(grocer_i+6))
   end
end
 
if grocer_z == [] then
// the function needs values for z and 0 ; A is set to 0
// so that it does not affect the estimation
   grocer_z=ones(grocer_n,1)
   A=0
end
 
// Do maximum likelihood estimation
select grocer_meth
case 'maxlik' then
   oresult = maxlik(filter_like,grocer_param,grocer_func,...
      grocer_y,grocer_x,grocer_F,grocer_z,varargin(:))
   grocer_like = -(oresult('f')+0.5*grocer_n*log(2*%pi));
   grocer_param = oresult('b');
case 'optim' then
   grocer_ng=size(grocer_param,1)
   grocer_ong=ones(grocer_ng,1)
   grocer_ch='deff(''[f,g,ind]=grocer_cost(grocer_param,...
      ind)'',[''f=filter_like(grocer_param,'''''+grocer_func...
      +''''',grocer_y,grocer_x,grocer_F,grocer_z)'';...
      ''g=numz0(filter_like,grocer_param,...
      grocer_ng,grocer_ong,1e-5,'''''+grocer_func+''''',...
      grocer_y,grocer_x,grocer_F,grocer_z)''])'
   execstr(grocer_ch)
   [f,g]=grocer_cost(grocer_param)
   [grocer_like,grocer_param] = optim(grocer_cost,grocer_param)
   grocer_like=-(grocer_like+0.5*grocer_n*log(2*%pi));
end
[u,s,v]=svd(hessian(filter_like,grocer_param,...
      grocer_func,grocer_y,grocer_x,grocer_F))
grocer_vcovt=u * diag(ones(size(s,1),1) ./ diag(s)) *v'
 
// retrieve Q, R , A and, eventually, grocer_priorb0
execstr(grocer_func)
 
// retrieve filtered beta and sigmas
[betat,betaf,sigmatt,sigmatf]=...
filter(grocer_param,grocer_func,grocer_y,grocer_x,grocer_F,...
grocer_z)
 
tmp=sqrt(diag(grocer_vcovt))
tstat = grocer_param ./ tmp;
yhat = zeros(grocer_n,1);
for i = 1:grocer_n
  yhat(i) = grocer_x(i,:)*betat(i,:)'+grocer_z(i,:)*A;
end
 
resid = grocer_y-yhat;
 
// retrieve smoothed beta and sigmas
[betas,sigmats]=smoothing(grocer_F,betat,betaf,sigmatt,sigmatf,...
grocer_begsmooth)
 
// return results structure information
rkalman=tlist(['results';'meth';'Q';'R';'A';'F';'priorb0';...
'priorv0';'betat';'betaf';'betas';'sigmatt';'sigmatf';...
'sigmats';'param';'vcov';'tstat';'y';'x';'z';'yhat';...
'resid';'like';'nobs';'nvar'],...
'kalman',Q,R,A,grocer_F,grocer_priorb0,grocer_priorv0,betat,betaf,betas,...
sigmatt,sigmatf,sigmats,grocer_param,grocer_vcovt,tstat,...
grocer_y,grocer_x,grocer_z,yhat,resid,grocer_like,...
grocer_n,grocer_k)
endfunction
