function res=ms_estimate(grocer_y,grocer_x,grocer_z,grocer_transf,grocer_MS_typmod,grocer_MS_M,grocer_MS_M_V,grocer_MS_var_opt,grocer_MS_apriori,grocer_prt,grocer_init_beta_id,grocer_init_beta_co,grocer_init_prob,grocer_init_var)
 
// PURPOSE: estimate a MSVAR model by the maximum likelihood
// method
// ------------------------------------------------------------
// INPUT:
// * grocer_y = (T x K) matrix of endogenous variables
// * grocer_x = (T x n_x) matrix of switching exogenous
//   variables
// * grocer_z = (T x n_z) matrix of non switching exogenous
//   variables
// * grocer_transf = the way data are transformed for the
//   estimation:
//   - 'none' if there is no transformation
//   - 'dem' if the data are demeaned
//   - 'stu' if the data are studentized
// * grocer_MS_typmod= type of MS model
//   - 1: mean-variance switching model
//   - 2: MS VAR regime dependent model
//   - 3: MS VAR intercept regime dependent model
//   - 4: partially regime dependent MS regression model
//   - 5: regime dependent MS regression model
// * grocer_MS_M = a scalar equal the # of states
// * grocer_MS_V = a scalar:
//    - 1 if the variance of the residuals is the same for all
//    states
//    - grocer_MS_M if the variance of the residuals differs
//    among the states
// * grocer_MS_var_opt = a scalar:
//    - 1 if the variance of residuals is heteroskedastic
//    - 2 if the variance of residuals is homoskedastic
//    - 3 if the variance of residuals is unconstrained
// * grocer_MS_apriori =
//    - 0 if there is no a priori datation
//    - 1 if there is an priori datation
// * grocer_prt = %t if the initial values of the parameters
//   are to be printed
// ------------------------------------------------------------
// OUTPUT:
// r = a results tlist with:
// * r('meth') = model literal type ('ms mean' 'ms var' or
//   'ms regression')
// * r('typmod') = model numbered type
// * r('y') = a (N x M) matrix of original endogenous variables
// * r('x') = the (N x n_x) matrix of exogenous switching
//   regressors
// * r('z') = the (N x n_z) matrix of exogenous non switching
//   regressors
// * r('ymat') = the (N x M) matrix of -transformed- endogenous
//   variables
// * r('xmat') = the (N x M) matrix of -transformed- exogenous
//   switching regressors
// * r('zmat') = the (N x M) matrix of -transformed- non
//   switching regressors
// * r('switching V') = a scalar:
//   - 1 if the variance does not switch with the states
//   - M if the variance switches with the states
// * r('var_opt') = a scalar:
//    - 1 if the variance of residuals is heteroskedastic
//    - 2 if the variance of residuals is homoskedastic
//    - 3 if the variance of residuals is unconstrained
// * r('nobs') = the # if observations
// * r('nendo') = the # of endogenous variables
// * r('nb_states') = the # of states
// * r('coeff') = the (np x 1) vector of parameters
// * r('llike') = the log-likekihood
// * r('grad') = the gradient at the solution
// * r('yhat') = the adjusted y
// * r('resid') = the residuals of the regression
// * r('dll') = the degrees of freedom
// * r('prob_st') = the (M x 1) vector of egodic state
//   probabilities
// * r('ptrans') = the (M x M) matrix of transition
//   probabilities
// * r('sigma') = the (M*M_V x M) variance-covariance matrix of
//   the residuals
// * r('beta_id') = the (1 x n_x*K*M) vector of switching
//   parameters
// * r('beta_co') = the (1 x n_z*K) vector of non switching
//   parameters
// * r('inv_sigma') = the (K x K) inverse of the variance
//   matrix
// * r('det_inv_sigma') = the determinant of the inverse of the
//   variance matrix
// * r('smoothed probs') = the (T x M) vector of smoothed
//   probabilities
// * r('stderr') = the (np x 1) vector of coefficients standard
//   errors
// * r('tstat') = the (np x 1) vector of associated t-stats
// * r('pvalue') = the (np x 1) vector of associated p-values
// * r('covbeta') = the (np x np) variance-covariance matrix of
//   the parameters
// * r('corbeta') = the (np x np) correlation matrix of the
//   parameters
// * r('ptrans_tstat') = the (M x 1) vector of t-stats for the
//   transition probabilities
// * r('beta_id_tstat') = the (1 x n_x*K*M) vector of t-stats for
//   switching parameters
// * r('beta_co_tstat') = the (1 x n_z*K) vector of t-stats for
//   non switching parameters
// * r('sigma_tstat') = the (M*M_V x M) matrix of t-stats for the
//   variance-covariance matrix of the residuals
// * r('ptrans_pvalue') = the (M x M) matrix of t-stats for
//   transition probabilities
// * r('beta_id_pvalue') = the (1 x n_x*K*M) vector of t-stats for
//   switching parameters
// * r('beta_co_pvalue') = the (1 x n_z*K) vector of t-stats for
//   non switching parameters
// * r('sigma_pvalue') = the (M*M_V x M) matrix of t-stats for
//   the variance-covariance matrix of the residuals
// ------------------------------------------------------------
// Adapted to Scilab by Eric Dubois
// http://dubois.ensae.net/grocer.html
// from Gauss library MSVarlib 2.0
// Copyright (C) 2004 by Benoit BELLONE.
// http://bellone.ensae.net
// All rights reserved
 
 
 
y_mat=MSVAR_Sample(grocer_y,grocer_transf);
x_mat=MSVAR_Sample(grocer_x,grocer_transf);
z_mat=MSVAR_Sample(grocer_z,grocer_transf);
 
[grocer_MS_T,grocer_MS_K]=size(y_mat)
grocer_MS_n_x=size(x_mat,2);
grocer_MS_n_z=size(z_mat,2);
 
if grocer_MS_M_V==1 then
   grocer_MS_MlikeMV=ones(1,grocer_MS_M)
else
   grocer_MS_MlikeMV=1
end
 
if grocer_MS_typmod == 1 then
   meth='ms mean'
elseif grocer_MS_typmod == 2 | grocer_MS_typmod == 3 then
   meth='ms var'
elseif grocer_MS_typmod == 4 | grocer_MS_typmod == 5 then
   meth='ms regression'
end
 
// Automatic Initialization ================================
[param] = MSVAR_SetInit(y_mat,x_mat,z_mat,grocer_MS_M,grocer_MS_var_opt,grocer_MS_M_V,grocer_MS_typmod,grocer_MS_apriori,grocer_init_beta_id,grocer_init_beta_co,grocer_init_prob,grocer_init_var);
 
if grocer_prt then
   disp("Initial statistical moments and parameters")
//   MSVAR_PrintMoment(y_mat,grocer_MS_M,grocer_MS_M_V,grocer_init_beta_id,grocer_init_beta_co,grocer_init_var);
   MSVAR_PrintInit(param,grocer_MS_K,grocer_MS_M,grocer_MS_var_opt,grocer_MS_M_V,grocer_MS_n_x,grocer_MS_n_z);
end
 
// ====================Maximisation of Log Likelihood ====================  */
disp(" Optimization step, be patient ...");
disp(' ')
disp(' ')
grocer_Vec_Mat_foo=MSVAR_Vec_Mat
 
[paramfin,likl1,grad] = MSVAR_MaxHmm(param);
 
// recover the true values of the parameters
//parameter=MSVAR_Constraint(paramfin)
nbparam=size(paramfin,1);
dll=grocer_MS_T-nbparam;
 
// explode the vector of parameters into their various
// dimensions
[prob_st,ptrans,beta_id,beta_co,sigma,inv_sigma,det_inv_sigma]=...
MSVAR_Vec_Mat(paramfin,grocer_MS_K,grocer_MS_M,grocer_MS_var_opt,grocer_MS_M_V,grocer_MS_n_x,grocer_MS_n_z);
beta_co=beta_co(:,1:grocer_MS_K)
 
// recovers some useful output
[Likv,y_hat,resid,PR,PR_STT,PR_STL] = MSVAR_Filt(paramfin)
 
// calculate the smoothed probabilities
PR_smoo = MSVAR_smooth(PR_STT,PR_STL,ptrans);
 
res=tlist(['results';'meth';'typemod';'y';'x';'z';'ymat';'xmat';'zmat';'switching V';...
'var_opt';'nobs';'nendo';'nb_states';'coeff';'llike';'grad';'yhat';'resid';'dll';'prob_st';...
'ptrans';'sigma';'beta_id';'beta_co';'inv_sigma';'det_inv_sigma';'filtered probs';'smoothed probs';'stderr';...
'tstat';'pvalue';'covbeta';'corbeta';'ptrans_tstat';'beta_id_tstat';'beta_co_tstat';...
'sigma_tstat';'ptrans_pvalue';'ptrans_pvalue';'beta_id_pvalue';'beta_co_pvalue';'sigma_pvalue'],...
meth,grocer_MS_typmod,grocer_y,grocer_x,grocer_z,y_mat,x_mat,z_mat,grocer_MS_M_V,grocer_MS_var_opt,...
grocer_MS_T,grocer_MS_K,grocer_MS_M,paramfin,likl1,...
grad,y_hat,resid,dll,prob_st,ptrans,sigma,beta_id,beta_co,inv_sigma,det_inv_sigma,...
PR_STT,PR_smoo)
 
res=MSVAR_stderr(res,grocer_hdelta)
 
endfunction
