function res=ms_forecast(grocer_rms,grocer_hprev,grocer_exo_com,grocer_exo_idio,grocer_prt)
 
// PURPOSE: forecast with a MSVAR model
// ------------------------------------------------------------
// INPUT:
// * grocer_rms = a tlist result from a ms switching estimation
// * grocer_hprev = the forecast period which can be either
//   . a [n1 n2] constant vector where n1 and n2 are the lead
//     over the last period of the estimation (n1<=0 means that
//     the forecast begins within the estimation period)
//   . a n constant which is equivalent to [1 n] (forecast
//     begins just after the estimation period)
//   . a [n1 n2] string vector where n1 and n2 are the time
//     periods for forecasting (a possibility open only if the
//     MS regression has been performed with ts)
//   . a n string which is equivalent to [1 n] (forecast
//     begins just after the estimation period)
// * grocer_exo_com = the data on the forecast horizon for the
//   non switching exogenous variables that can be:
//   . a (H x n_x) matrix with H the number of forecasts and
//     n_x the number of non switching exogenous variables
//   . a list of ts available over the forecasting horizon
//   . a list of (H x 1) vectors and ts available over the
//     forecasting horizon
//   . a (H x n_x) string matrix of names
//   (note: when the ms tlist results comes from a ms-mean
//    or a var estimation, the variables are useless and can
//    therefore be omitted)
// * grocer_exo_idio = the data on the forecast horizon for the
//   switching exogenous variables that can be:
//   . a (H x n_z) matrix with H the number of forecasts and
//     n_z the number of switching exogenous variables
//   . a list of ts available over the forecasting horizon
//   . a list of (H x 1) vectors and ts available over the
//     forecasting horizon
//   . a (H x n_x) string matrix of names
//   (note: when the ms tlist results comes from a ms-mean
//    or a var estimation, the variables are useless and can
//    therefore be omitted)
// ------------------------------------------------------------
// OUTPUT:
// res = a results tlist with:
// * res('meth') = 'msf'
// * res('r_ms') = the tlist results from the MS estimation
// * res('pstates') = the (H x nb_states) matrix of the
//   probabilities of the respective states over the forecasting
//   horizon
// * res('prev_states') = the matrix of forecast of the endogenous
//   variables across the states
// * res('prev') = the matrix of forecast of the endogenous
//   variables
// * res('hprev') = the forecast period
// ------------------------------------------------------------
// Copyright: Eric Dubois 2007
// http://dubois.ensae.net/grocer.html
 
 
 
[grocer_nargout,grocer_nargin]=argn(0)
if grocer_nargin <5 then
   grocer_prt=%t
elseif strsubst(grocer_prt,' ','') == 'noprint' then
   grocer_prt=%f
end
 
// calculate the range for forecasting
[grocer_n0,grocer_nf,grocer_hprev]=hprev2vec(grocer_rms,grocer_hprev)
 
if typeof(grocer_hprev) == 'string' then
   grocer_boundsvar=grocer_hprev
end
 
grocer_meth=grocer_rms('meth')
grocer_prests=grocer_rms('prests')
 
select grocer_meth
 
case 'ms mean' then
   grocer_exo_comp=[]
   grocer_exo_idiop=ones(grocer_nf-grocer_n0+1,1)
 
case 'ms regression' then
   [grocer_exo_comp,grocer_name1,grocer_exo_idiop,grocer_name2,grocer_junk,grocer_b]=explouniv(grocer_exo_com,grocer_exo_idio,[],['exo_com';'idio_com'])
   z=grocer_rms('z')
// transform the exogenous variables over the forecasting period in accordance with the
// transformation of the original variables
 
   zmat=grocer_rms('zmat')
   if sum(abs(zmat)) ~= 0 then
      dzm=mean(z-zmat,'r')
      pente_zm=(z(1,:)-dzm) ./ zmat(1,:)
      grocer_exo_comp=(grocer_exo_comp-(dzm .*. ones(size(grocer_exo_comp,1),1))) ./ (pente_zm .*. ones(size(grocer_exo_comp,1),1))
   else
      grocer_exo_comp=zeros(grocer_nf-grocer_n0+1,1)
   end
   x=grocer_rms('x')
   xmat=grocer_rms('xmat')
   if sum(abs(xmat)) ~= 0 then
      dxm=mean(x-xmat,'r')
      pente_xm=(x(1,:)-dxm )  ./ xmat(1,:)
      grocer_exo_idio=(grocer_exo_idiop-(dxm .*. ones(size(grocer_exo_idiop,1),1))) ./ (pente_xm .*. ones(size(grocer_exo_idiop,1),1))
   else
      grocer_exo_idio=zeros(grocer_nf-grocer_n0+1,1)
   end
else
   if grocer_meth ~= 'ms var' then
      error(grocer_meth+' is not an available type for first arg in ms_forecast')
   end
 
   if grocer_n0 > 1 then
      disp('forecasts will be stored from the '+string(grocer_n0)+...
      ' th period after the estimation period, but forecasting will start ...
       immediately after the estimation period')
   end
 
end
 
// recover useful output from the results tlist
ptrans=grocer_rms('ptrans')
psmoothed=grocer_rms('smoothed probs')
nobs=grocer_rms('nobs')
nendo=grocer_rms('nendo')
nb_states=grocer_rms('nb_states')
beta_co=grocer_rms('beta_co')
beta_id=grocer_rms('beta_id')
 
// if the user wants only forecasts from an horizon far in the future,
// then define the start of the underlying forecast at date T+1
grocer_startfore=min(1,grocer_n0)
pstates=zeros(grocer_nf-grocer_startfore+2,nb_states)
pstates(1,:)=psmoothed(nobs+grocer_startfore-1,:)
 
for i=grocer_startfore:grocer_nf
// update probas of each states using the probabilities of transition
   pstates(i-grocer_startfore+2,:)=pstates(i-grocer_startfore+1,:)*ptrans'
end
// remove the last observed proba used to feed the forecasted ones
pstates=pstates(grocer_n0-grocer_startfore+2:grocer_nf-grocer_startfore+2,:)
 
if grocer_meth == 'ms var' then
   nlag=grocer_rms('nlag')
   ys=zeros((grocer_nf-grocer_startfore+1)*nendo,nb_states)
   y=zeros((grocer_nf-grocer_startfore+1),nendo)
 
   // recover the last observed values to feed the var
   select grocer_rms('typemod')
 
   case 2 then
   // all coeffs are regime dependent
      yp=matrix(grocer_rms('ymat')($-grocer_startfore+1:-1:$-grocer_startfore+2-nlag,:)',1,-1)
 
      for i=grocer_startfore:grocer_nf
         ys((i-grocer_startfore)*nendo+[1:nendo],:)=matrix([yp 1]*beta_id,nendo,-1)
         y(i-grocer_startfore+1,:)=sum(ys(i-grocer_startfore+1,:) .* pstates(i-grocer_startfore+1,:))
         yp=[y(i-grocer_startfore+1,:) yp(nendo+1:$)]
      end
 
   case 3 then
 
   // only the const is regime dependent
      yp=matrix(grocer_rms('ymat')($-grocer_n0+1:-1:$-grocer_n0+2-nlag,:)',1,-1)
      beta_idt=matrix(beta_id,nendo,nb_states)
 
      for i=grocer_startfore:grocer_nf
         ys((i-grocer_startfore)*nendo+[1:nendo],:)=(beta_co'*yp') .*. ones(1,nb_states)+beta_idt
         y(i-grocer_startfore+1,:)=pstates(i-grocer_startfore+1,:)*ys((i-grocer_startfore)*nendo+[1:nendo],:)'
         yp=[y(i-grocer_startfore+1,:) yp(nendo+1:$)]
      end
   end
   // remove the forecasts that do not interest the user
   ys([1:grocer_n0-1]*nendo,:)=[]
   y(1:grocer_n0-1,:)=[]
 
else
   ys=(grocer_exo_comp*beta_co) .*. ones(1,nb_states)+grocer_exo_idiop*beta_id
 
   y_weighted=ys .* (pstates .*. ones(1,nendo))
   y=y_weighted*(ones(nb_states,1) .*. eye(nendo,nendo))
end
 
res=tlist(['results';'meth';'r_ms';'probf';'prev_states';'forecast';'h forecast'],'msf',grocer_rms,pstates,ys,y,grocer_hprev)
 
if grocer_prests then
// transform the matrix of forecasts into ts and store them into
// the results tlist
   for i=1:nendo
      execstr('res(1)($+1)='''+grocer_rms('namey')(i)+'_f''')
      execstr('res($+1)=reshape(y(:,'+string(i)+'),'''+grocer_hprev(1)+''')')
   end
 
else
// transform the matrix of forecasts into vectors and store them into
// the results tlist
  for i=1:nendo
     execstr('res(1)($+1)='''+grocer_rms('namey')(i)+'_f''')
     execstr('res($+1)=y(:,'+string(i)+')')
  end
 
end
 
if grocer_prt then
   prtvarf(res,%io(2))
end
 
endfunction
 
