function [rsur]=surregu(grocer_namecoef,grocer_speccara,grocer_speccarb,grocer_crit,grocer_dropna,varargin)
 
// PURPOSE: provide Zellner Seemingly Unrelated Regression in
// the standard case when all equations are estimated on the
// same period (subroutine of function sur)
// ------------------------------------------------------------
// INPUT:
// * grocer_namecoef = the names of the coefficients
// * grocer_speccara = column vector of characters that must be
//   found after the name of a coefficient
// * grocer_speccarb = column vector of characters that must be
//   found before the name of a coefficient
// * grocer_crit = a scalar, the convergence criterion
//   (optional; default =1e-4)
// * varargin = equations written
// * grocer_drpna = 'dropna' if the user wants to remove the
//     NA values from the data
// 'vary=coef1*varx1+...+coefi*varxi'
//   where:
//   - coefi = the name of a coefficient
//   - varxi = the name of a variable
// ------------------------------------------------------------
// OUTPUT:
// rsur=a tlist with
// - rsur('meth') = 'sur'
// - rsur('nobs') = # of observations
// - rsur('neqs') = # of estimated equations
// - rsur('ncoef') = # of estimated coefficients
// - rsur('beta') = bhat
// - rsur('tstat') = t-stats
// - rsur('pvalue') = pvalue of the betas
// - rsur('sigma') = covariance matrix of the residuals
// - rsur('sigu') = (1 x neqs) sum of squared residuals
// - rsur('sigu') = (1 x neqs) sum of squared residuals
// - rsur('dw') = (1 x neqs) Durbin-Watson
// - rsur('prests') = boolean indicating the presence or
//     absence of a time series in the regression
// - rsur('namecoef') = (ncoef x 1) mame of the coeffcients
// - rsur('namey') = name of endogenous variables
// - rsur('eqs') = list of the neqs equations
// - rsur('coefs') = list of the coefs names in each equation
// - rsur('bounds') = the bounds of all equations
// - rsur('dropna') = boolean indicating if NAs had
//		    been dropped
// - rsur('nonna') = vector indicating position of
//     non-NA values (if the option 'dropna' was active)
// ------------------------------------------------------------
// Copyright: Eric Dubois 2006-2007
// http://dubois.ensae.net/grocer.html
 
// explode the equation into the x,y matrices, the name of the
// endogenous variables, the list of indexes of the coefficients
// in each equation and the bounds and the boolean indicating
// the presence of a ts
 
 
 
 
[x,y,grocer_boundsvarb,grocer_prests,listcoef,namey,ncoefeqs,nonna]=...
explosys(grocer_namecoef,grocer_speccara,grocer_speccarb,%f,grocer_dropna,varargin(:))
 
convg=grocer_crit*2
// uses results from ols as starting values
beta0=ols0(y,x)
resid0=y-x*beta0
nbit=0
nobs=size(y,1)/grocer_neqs
ncoef=size(grocer_namecoef,1)
 
while (convg > grocer_crit & nbit < grocer_itmax) then
   nbit=nbit+1
   sigma=zeros(grocer_neqs,grocer_neqs)
   resd=resid0 ./ (sqrt(nobs-ncoefeqs).*.ones(nobs,1))
   rest=matrix(resd,nobs,grocer_neqs)
   sigma=rest'*rest
   sqrtsigma=sigma^(-0.5).*.eye(nobs,nobs)
   sqrtsigmax=sqrtsigma*x
   [bhat,ixpomegam1x]=ols0(sqrtsigma*y,sqrtsigmax)
 
   resid=y-x*bhat
   convg=sum(abs(bhat-beta0))
   beta0=bhat
   resid0=resid
end
 
resid=matrix(resid,nobs,grocer_neqs)
tstat=bhat./sqrt(diag(ixpomegam1x))
pvalue=zeros(ncoef,1)
ds=diag(sigma)
ser=sqrt(ds)
sigu=ds.*(nobs-ncoefeqs)
corm=var2cor(sigma)
 
for i=1:ncoef
   pvalue(i)=(1-cdfnor("PQ",abs(tstat(i)),0,1))*2
end
 
// durbin-watson
dw=zeros(1,grocer_neqs)
for i=1:grocer_neqs
   ediff = resid(2:nobs,i)-resid(1:nobs-1,i)
   dw(i) = ediff'*ediff/sigu(i)
end
 
rsur=tlist(['results';'meth';'nobs';'neqs';'ncoef';'beta';...
'tstat';'pvalue';'resid';'sigma';'corr';'sigu';'ser';'dw';...
'prests';'dropna';'namecoef';'namey';'eqs';'coefs'],...
'sur',nobs,grocer_neqs,ncoefeqs,bhat,tstat,pvalue,...
resid,sigma,corm,sigu,ser,dw,grocer_prests,grocer_dropna,...
grocer_namecoef,namey,varargin,listcoef)
 
if grocer_prests then
   rsur(1)($+1) = 'bounds'
   rsur('bounds')=grocer_boundsvarb
end
 
if grocer_dropna then
   rsur(1)($+1)='nonna'
   rsur('nonna')=nonna
end
 
endfunction
