function [result]=dfp_min(grocer_func,grocer_pin,varargin)
 
// PURPOSE: DFP minimization routine to minimize func
// (Converted from Numerical Recipes book dfpmin routine)
// ------------------------------------------------------------
// INPUT:
// * grocer_func = (minus likelihood) function to minimize
// * grocer_pin = parameter vector fed to func
// * grocer_varargin = 'maxit=x' to set the maximum # of iterations
//              (default = 500)
//  and arguments of function func others that the parameters
// ------------------------------------------------------------
// OUTPUT:
// result = a result tlist with:
//   - result('meth') = 'dfp'
//   - result('b')    = (kx1) minimizing vector
//   - result('f')    = value of func at solution values
//   - result('hess') = hessian at solution values
//   - result('iter') = # number of iterations
//   - result('flag') = 0 for convergence, 1 for non-convergence
// ------------------------------------------------------------
// NOTE:
// func must take the form func(b,varargin)
// where:  b        = parameter vector (k x 1)
//         varargin = arguments passed to the function
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
// default options
 
 
grocer_tol = sqrt(%eps);
grocer_maxit = 500;
grocer_optprt = 0;
grocer_grad='numz0'
grocer_delta=1e-5
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if type(varargin(grocer_i)) == 10 then
      if (part(varargin(grocer_i),[1:5]) == 'maxit' ...
      | part(varargin(grocer_i),[1:5]) == 'delta' ...
      | part(varargin(grocer_i),[1:3]) == 'tol' ...
      | part(varargin(grocer_i),[1:6]) == 'optprt') ...
      then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i) = null()
      else
         if part(varargin(grocer_i),[1:4]) == 'grad' then
            grocer_ch=strsubst('grocer_'+varargin(grocer_i)...
            ,'=','=''')+''''
            execstr(grocer_ch)
            varargin(grocer_i)=null()
         end
      end
   end
end
 
if grocer_grad == 'numz0' then
   grocer_chg='numz0(grocer_func,grocer_pin,...
   grocer_n,grocer_in,grocer_delta,varargin(:))'
else
   grocer_chg=grocer_grad+'(grocer_pin)'
end
 
grocer_epss = sqrt(%eps);
 
grocer_n= size(grocer_pin,1);
grocer_in=ones(grocer_n,1)
 
grocer_fp=grocer_func(grocer_pin,varargin(:))
grocer_g=evstr(grocer_chg)
 
grocer_xi = -grocer_g
grocer_hessn = eye(grocer_n,grocer_n);
grocer_iter = 1;
 
 
while grocer_iter<=grocer_maxit then
 
  [grocer_pin grocer_fret] = linmin(grocer_pin,...
     grocer_xi,grocer_tol,grocer_func,varargin(:))
 
  if grocer_optprt==1 then
    // print iteration results
     grocer_mat2prt = ['iter #' 'function value' ; ...
     string(grocer_iter) string(grocer_fp) ];
     printmat(grocer_mat2prt,%io(2))
     grocer_mat2prt = ['parameter value' 'gradient' ; ...
     string(grocer_pin) string(grocer_g)];
     printmat(grocer_mat2prt,%io(2))
  end
 
  if 2*abs(grocer_fp-grocer_fret)<=grocer_tol*(abs(grocer_fp)+...
     abs(grocer_fret)+grocer_epss) then
    grocer_pout = grocer_pin;
    grocer_fout = grocer_fret
    grocer_hessf=hessian(grocer_func,grocer_pin,varargin(:))
    result = tlist(['resulmin';'meth';'b';'f';'hess';'iter';'flag'],...
            'dfp',grocer_pin,grocer_fret,grocer_hessf,grocer_iter,0)
    return
 
  end
 
  grocer_fp=grocer_func(grocer_pin,varargin(:))
  grocer_dg = grocer_g
  grocer_fret = grocer_func(grocer_pin,varargin(:))
 
  grocer_g = evstr(grocer_chg)
 
  grocer_dg = grocer_g-grocer_dg;
  grocer_hdg = grocer_hessn*grocer_dg;
  grocer_fac = grocer_dg'*grocer_xi;
  grocer_fae = grocer_dg'*grocer_hdg;
 
  if grocer_fac==0 then
    grocer_pout = grocer_pin;
    grocer_fout = grocer_fret;
    grocer_hessf=hessian(grocer_func,grocer_pin,varargin(:))
    result = tlist(['resulmin';'meth';'b';'f';...
       'hess';'iter';'flag'],'dfp',grocer_pin,...
        grocer_fret,grocer_hessf,grocer_iter,0)
    return	
     	
  else
    grocer_fac = 1/grocer_fac;
  end
 
  if grocer_fae==0 then
    grocer_pout = grocer_pin;
    grocer_fout = grocer_fret;
    grocer_hessf=hessian(grocer_func,grocer_pin,varargin(:))
    result = tlist(['resulmin';'meth';'b';'f';...
       'hess';'iter';'flag'],'dfp',grocer_pin,...
        grocer_fret,grocer_hessf,grocer_iter,0)
    return
 
  else
    grocer_fad = 1/grocer_fae;
  end
 
  grocer_dg = grocer_fac*grocer_xi-grocer_fad*grocer_hdg;
  grocer_hessn = grocer_hessn+grocer_fac*grocer_xi*...
     grocer_xi'-grocer_fad*grocer_hdg*grocer_hdg'+...
     grocer_fae*grocer_dg*grocer_dg';
  grocer_xi = -grocer_hessn*grocer_g
  grocer_iter = grocer_iter+1;
end
 
grocer_hessf=hessian(grocer_func,grocer_pin,varargin(:))
result = tlist(['results';'meth';'b';'f';...
      'hess';'iter';'flag'],'dfp',grocer_pin,...
      grocer_fret,grocer_hessf,grocer_iter,0)
endfunction
