function [result]=frpr_min(grocer_func,grocer_pin,varargin)
 
// PURPOSE: Fletcher,Reeves,Polak,Ribiere minimization routine
// to minimize func (Converted from Numerical Recipes
// book frprmn routine)
// ------------------------------------------------------------
// INPUT:
// * grocer_func = (minus likelihood) function to minimize
// * grocer_pin = parameter vector fed to func
// * varargin = 'maxit=x' to set the maximum # of iterations
//              (default = 500)
// and arguments of function grocer_func others that the
// searched parameters
// ------------------------------------------------------------
// OUTPUT:
// result = a result list with:
//   - result('meth') = 'dfp'
//   - result('b')    = (kx1) minimizing vector
//   - result('f')    = value of func at solution values
//   - result('hess') = hessian at solution values
//   - result('iter') = # number of iterations
//   - result('flag') = 0 for convergence, 1 for non-convergence
// ------------------------------------------------------------
// NOTE:
// grocer_func must take the form grocer_func(b,varargin)
// where:  b        = parameter vector (k x 1)
//         varargin = arguments passed to the function
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
// default options
 
 
grocer_tol = sqrt(%eps);
grocer_maxit = 500;
grocer_pflag = 0;
grocer_grad='numz'
grocer_delta=1e-5
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      if (part(varargin(grocer_i),[1:5]) == 'maxit' ...
      | part(varargin(grocer_i),[1:5]) == 'delta' ...
      | part(varargin(grocer_i),[1:5]) == 'pflag' ...
      | part(varargin(grocer_i),[1:3]) == 'tol' ...
      | part(varargin(grocer_i),[1:6]) == 'optprt') ...
      then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i) = null()
      else
         if part(varargin(grocer_i),[1:4]) == 'grad' then
            grocer_ch=strsubst('grocer_'+varargin(grocer_i),...
               '=','=''')+''''
            execstr(grocer_ch)
            varargin(grocer_i)=null()
         end
      end
   end
end
 
if grocer_grad == 'numz' then
   grocer_chg='numz0(grocer_func,grocer_pin,grocer_n,...
      grocer_in,grocer_delta,varargin(:))'
else
   grocer_chg=grocer_grad+'(grocer_pin)'
end
 
grocer_epss = sqrt(%eps);
 
grocer_n = size(grocer_pin,1);
grocer_in=ones(grocer_n,1)
 
grocer_deltagrad = grocer_tol;
grocer_fp = grocer_func(grocer_pin,varargin(:))
grocer_xi = evstr(grocer_chg)
grocer_g = -grocer_xi;
grocer_h = grocer_g;
grocer_xi = grocer_h;
 
grocer_iter = 0;
 
while grocer_iter<=grocer_maxit then
  grocer_iter = grocer_iter+1;
 
   [grocer_pin grocer_fret] = linmin(grocer_pin,...
   grocer_xi,grocer_tol,grocer_func,varargin(:))
 
  if grocer_pflag==1 then
     // print iteration results
     grocer_mat2prt = ['iter #' 'function value' ; ...
        string(grocer_iter) string(grocer_fp) ];
     printmat(grocer_mat2prt,%io(2))
     grocer_mat2prt = ['parameter value' 'gradient' ; ...
        string(grocer_pin) string(grocer_xi)];
     printmat(grocer_mat2prt,%io(2))
  end
 
  if 2*abs(grocer_fp-grocer_fret)<=grocer_tol*(abs(grocer_fp)...
     +abs(grocer_fret)+grocer_epss) then
    grocer_hessf = hessian(grocer_func,grocer_pin,varargin(:))
    result = tlist(['resulmin';'meth';'b';'f';'hess';'iter';'flag'],...
       'frpr',grocer_pin,grocer_fret,grocer_hessf,grocer_iter,0)
    return
 
  end
  grocer_fp = grocer_func(grocer_pin,varargin(:))
  grocer_xi = evstr(grocer_chg)
 
 
  grocer_gg = grocer_g'*grocer_g;
  grocer_dgg = ((grocer_xi+grocer_g)')*grocer_xi;
 
  if grocer_gg==0 then
    grocer_hessf = hessian(grocer_func,grocer_pin,varargin(:))
    result = tlist(['resulmin';'meth';'b';'f';'hess';'iter';'flag'],...
       'pow',grocer_pin,grocer_fret,grocer_hessf,grocer_niter,0)
    return
  end
 
  grocer_gam = grocer_dgg/grocer_gg;
  grocer_g = -grocer_xi
  grocer_h = grocer_g+grocer_gam*grocer_h;
  grocer_xi = grocer_h;
 
end
// end of while
 
hessf = evstr(chg)
result = tlist(['resulmin';'meth';'b';'f';'hess';'iter';'flag'],...
            'pow',grocer_pin,grocer_fret,hessf,grocer_iter,1)
 
endfunction
