function [theta,x_T,P_T,ytT,grocer_theta2mat,grocer_e4param,lab,ineq,f,g,u0,n1,AR,ARS,MA,MAS,G,V,mat2thet]=varma1(y0,AR,ARS,MA,MAS,v,s,grocer_E4OPTION,inite4,nap,iter,G,exo0,namexo)
 
// PURPOSE: estimate a VARMA model using E4 functions
// the ARMA model has the following form:
// AR(L)*ARS(L^s) y = MA(L)*MAS(L^s) e [+G(L)X]
// where L is the lag operator, X is an optional vector of
// exogenous variables
// ------------------------------------------------------------
// INPUT:
// * endo = either
//   - a matrix of strings, each one being the name of a
//   variable
//   - a (Txn) real matrix
//   - a ts
//   - a list of variables
//   each element could be a timeseries, a real vector,
//   a real matrix or a string (the name of a variable with
//   one of the types cited above, between quotes)
// * AR = the matrix [] or a (n x (n*p)) matrix
//   with:
//   - n = # of endogenous variables in endo
//   - p = # of lags in the AR part of the process
// * ARS = the matrix [] or a (n x (n*ps)) matrix
//   with ps = # of lags in the seasonal AR part of the
//   process
// * MA = the matrix [] or a (n x (n*q)) matrix
//   with: q = # of lags in the AM part of the process
// * MAS = the matrix [] or a (n x (n*qs)) matrix
//   with qs = # of lags in the seasonal MA part of the process
// * v = a (nx1) vector if the user wants to impose
//   independence between resisduals or a (nxn) matrix in the
//   other case
// * s = a scalar representing the order of the
//   seasonality
// * varargin = optional arguments that can be:
//   - 'init=own' if the users wants to use as starting values
//   of the optimisation algorithm those given by her as entry
//   of varma
//   - 'exo=xxx' where xxx is the names of exogenous variables
//   which can take the form of a list or a vector of strings
//   - 'Gexo=xxx' where xxx is a vector of coefficients
//   corresponding to the exogenous variables
//   - 'delta=xx' where xx is the increment used to calculate
//   the numerical derivative of the log-likelihood
//   (default: sqrt(%eps)
//   - 'noprint' if the user does not want to print the results
// ------------------------------------------------------------
// OUTPUT:
// result = a results tlist with:
// - result('meth') ='varma'
// - result('y') = (nobsxnendo) matrix of values for the
//   endogenous variables
// - result('namey') = (nvarx1) vector of names for the
//   endogenous variables
// - result('z') = (nobsx(nendo+nexo)) matrix of endogenous and
//   exogenous variables
// - result('nobs') = # of observations
// - result('nendo') = # of endogenous variables
// - result('nvar') = # of endogenous and exogenous variables
// - result('coeff') = (npx1) vector of estimated parameters
// - result('lab') = (npx1) string vector of names for the
//   estimated parameters
// - result('llike') = log-likelihood of the model
// - result('grad') = gradient of the log-likelihood of the
//   model at the estimated parameters
// - result('tstat') = Student's t of the coefficients
// - result('std') = (npx1) Student's t of the coefficients
// - result('exact') = a boolean equal to %T indicatinge that
//   the covariance matrix of the estimated parameters has been
//   calculated with an exact formula
// - result('corr') = (npxnp) correlation matrix
// - result('AIC') = Akaike information criterion
// - result('BIC') = Schwartz information criterion
// - result('theta2mat') = (npx1) string vector making the
//   transformation of the vector of estimated parameters into
//   the matrices of the problem
// - result('seas') = order of the seasonality
// - result('nexo') = # of exogenous variables in the model
// - result('resid') = (nobsx1) vector of filtered residuals
// - result('AR') = the estimated AR part of the ARMA
// - result('MA') = the estimated MA part of the ARMA
// - result('ARS') = the estimated seasonal AR part of the
//   ARMA
// - result('MAS') = the estimated seasonal MA part of the
//   ARMA
// - result('V') = the estimated variance of residuals
// - result('G') = the estimated variance of residuals
// - result('k') = maximum degree of the total AR, MA and G
//   parts
// - result('p') = # of AR parameters
// - result('P') = # of ARS parameters
// - result('q') = # of MA parameters
// - result('Q') = # of MAS parameters
// - result('lagexo') = # of lags applied to the vector of
//   exogenous variables
// - result('n') = n*nendo
// - result('type') = 1 = type of the e4 model
// - result('innov') = a flag indicating that this is an
//   innovation model (a e4 parameter)
// - result('prests') = a boolean indicating whether there are
//   ts in the regression
// - result('E4OPTION') = the list of options used to perform
//   optimisation (see sete4opt for details)
// - result('yhat') = adjusted y
// - result('resid') = residuals of the VARMA
// - result('std resid') = standardized residuals of the VARMA
// - result('exo') = value of the exogenous variables (if any)
// - result('exo') = name of the exogenous variables (if any)
// - result('bounds') = bounds of the estimation (if there are
//   ts in the regression)
// ------------------------------------------------------------
// Copyright: Eric Dubois 2007
// http://dubois.ensae.net/grocer.html
 
// set defaults
 
 
[nargout,nargin]=argn(0)
grocer_e4_userflag=0
delta=sqrt(%eps)
grocer_theta2ss=theta2sss
[T,m]=size(y0)
 
labels=''+['AR' 'ARS' 'MA' 'MAS']
execstr(labels+'0='+labels)
 
for k=1:4
   obj=evstr(labels(k))
   [nr,nc]=size(obj)
   select typeof(obj)
   case 'constant' then
      execstr(labels(k)+'=0*obj')
   case 'string' then
      for i=1:nr
         for j=1:nc
            if isempty(strindex(obj(i,j),'=')) & isempty(strindex(obj(i,j),'<')) ...
            & isempty(strindex(obj(i,j),'>')) then
               execstr(labels(k)+'('+string(i)+','+string(j)+')=''0''')
            end
         end
      end
   end
end
 
[y,scale]=scalemat(y0)
V0=diag(10^(-scale))*varcov0(y0)*diag(10^(-scale))
[nr,nc]=size(v)
 
if or([nr nc] == 1) then
   V0=diag(V0)
end
 
if typeof(v) == 'constant' then
   v=V0
else
   for i=1:nr
      for j=1:nc
         if isempty(strindex(v(i,j),'=')) then
            execstr('v('+string(i)+','+string(j)+')=V0('+string(i)+','+string(j)+')')
         else
            v(i,j)=part(v(i,j),1:ind_eq)+string(evstr(part(v(i,j),1+ind_eq:length(v(i,j))))*sqrt(V0(i,i)*V0(j,j)))
         end
      end
   end
end
 
if nargin > 11 then
 
   [exo,scalex]=scalemat(exo0)
   u0=exo(1,:)
 
   [theta,grocer_theta2mat,lab,grocer_e4param,ineq,n1,mat2thet]=arma2param(m,AR,ARS,MA,MAS,v,0,s,G,size(exo0,2),namexo);
 
else
 
 
   exo0=[]
   exo=[]
   u0 =[]
 
   [theta,grocer_theta2mat,lab,grocer_e4param,ineq,n1,mat2thet]=arma2param(m,AR,ARS,MA,MAS,v,0,s);
end
 
grocer_e4param.T=T
 
z=[y exo]
 
if inite4 then
   // give good starting values
   [theta,f0]=e4prees1(theta,grocer_theta2mat,z)
   // recover the true variance (to understand why, see function theta2arm2)
   [AR,ARS,MA,MAS,V,G]=theta2arm2(theta,grocer_theta2mat)
 
//   theta2mat2=''''+grocer_theta2mat+''''
   if size(y,2) == 1 then
   // change AR, ARS, MA, MAS so that all roots are lower than 1
      corrv_ar=1
      corrv_ars=1
      corrv_ma=1
      corrv_mas=1
 
      if typeof(AR0) == 'cosntant' then
         [AR,corrv_ar]=transf_roots(AR)
      end
      if typeof(ARS0) == 'cosntant' then
         [ARS,corrv_ars]=transf_roots(ARS)
      end
      if typeof(MA0) == 'cosntant' then
         [MA,corrv_ma]=transf_roots(MA)
      end
      if typeof(ARS0) == 'cosntant' then
         [MAS,corrv_mas]=transf_roots(MAS)
      end
      V=sqrt(V*corrv_ar*corrv_ars/corrv_ma/corrv_mas)
 
      execstr(mat2thet)
   end
end
 
// create the function that gathers the estimate of the
// likelihood function and its derivative
str2foo='deff(''[f,g,ind]=llike(p,ind)'',...
   ''[f,g]=llike_innov(p,grocer_theta2mat,z,u0)'')'
execstr(str2foo)
 
if or(ineq(:,1) ~= -%inf) | or(ineq(:,2) ~= %inf) then
// there are inequality constraints
   [f,theta,g] = optim(llike,'b',ineq(:,1),ineq(:,2),theta);
else
// there is no inequality constraint
   [f,theta,g] = optim(llike,theta);
end
 
// recover the true variance (to understand why, see function theta2arm2)
[AR,ARS,MA,MAS,V,G]=theta2arm2(theta,grocer_theta2mat)
 
if size(y,2) == 1 then
// reestimate with all roots of AR, ARS, MA, MAS lower than 1
   corrv_ar=1
   corrv_ars=1
   corrv_ma=1
   corrv_mas=1
 
   if typeof(AR0) == 'constant' then
      [AR,corrv_ar]=transf_roots(AR)
   end
   if typeof(ARS0) == 'constant' then
      [ARS,corrv_ars]=transf_roots(ARS)
   end
   if typeof(MA0) == 'constant' then
      [MA,corrv_ma]=transf_roots(MA)
   end
   if typeof(ARS0) == 'constant' then
      [MAS,corrv_mas]=transf_roots(MAS)
   end
   V=V*(corrv_ar*corrv_ars/corrv_ma/corrv_mas)^2
 
   execstr(mat2thet)
 
end
 
 
[x_T,P_T,ytT]=smooth_innov(theta,grocer_theta2mat,z,u0)
 
// return to the original values and recover the true likelihood
V0=diag(10^scale)*V*diag(10^scale)
// let theta include the values for V
[N,S]=svd(V0)
V=N*sqrt(S)*N'
execstr(grocer_e4param.V2theta)
if grocer_e4param.r then
   G=diag(10^scale)*G*diag(10^scalex)
   execstr(grocer_e4param.G2theta)
   u0=diag(10^scalex)*u0
end
 
ytT=ytT .* (ones(T,1) .*. 10^scale)
z=[y0 exo0]
[f,g]=llike_innov(theta,grocer_theta2mat,z,u0)
V=V0
 
endfunction
 
